package ascass.parsing;

import org.jacop.constraints.Alldiff;
import org.jacop.constraints.And;
import org.jacop.constraints.Cumulative;
import org.jacop.constraints.Element;
import org.jacop.constraints.GCC;
import org.jacop.constraints.IfThen;
import org.jacop.constraints.Max;
import org.jacop.constraints.Min;
import org.jacop.constraints.Or;
import org.jacop.constraints.PrimitiveConstraint;
import org.jacop.constraints.Sum;
import org.jacop.constraints.XeqC;
import org.jacop.constraints.XeqY;
import org.jacop.constraints.XexpYeqZ;
import org.jacop.constraints.XgtC;
import org.jacop.constraints.XgtY;
import org.jacop.constraints.XgteqC;
import org.jacop.constraints.XgteqY;
import org.jacop.constraints.XltC;
import org.jacop.constraints.XltY;
import org.jacop.constraints.XlteqC;
import org.jacop.constraints.XlteqY;
import org.jacop.constraints.XmulYeqZ;
import org.jacop.constraints.XneqC;
import org.jacop.constraints.XneqY;
import org.jacop.constraints.XplusYeqZ;
import org.jacop.constraints.binpacking.Binpacking;
import org.jacop.core.IntVar;
import org.jacop.core.Store;
import org.jacop.search.ComparatorVariable;
import org.jacop.search.ExitChildListener;
import org.jacop.search.Indomain;
import org.jacop.search.LargestDomain;
import org.jacop.search.MaxRegret;
import org.jacop.search.MostConstrainedDynamic;
import org.jacop.search.MostConstrainedStatic;
import org.jacop.search.SmallestDomain;
import org.jacop.search.SmallestMax;
import org.jacop.search.SmallestMin;
import org.jacop.search.WeightedDegree;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import org.antlr.v4.runtime.ParserRuleContext;
import org.antlr.v4.runtime.misc.NotNull;
import org.antlr.v4.runtime.tree.ErrorNode;
import org.antlr.v4.runtime.tree.TerminalNode;
import org.jacop.constraints.Count;
import ascass.solving.MyIndomainHierarchical;
import ascass.solving.MyLds;
import ascass.solving.MyLimited;
import ascass.solving.VarVectors;

    
public class HcaspBaseListener implements HcaspListener {

    private Store store;
    private HashMap <String,IntVar> vars = new HashMap <>();
    private ExitChildListener <IntVar> exitChildListener = null;
    private final MyIndomainHierarchical valSelect = new MyIndomainHierarchical();
    private ComparatorVariable<IntVar> varSelect = null;
    private ComparatorVariable<IntVar> varSelectTieBreak = null;
    private ArrayList<IntVar> orderedVars = new ArrayList<>();
    private ArrayList<Integer> prios = new ArrayList<>();
    private boolean varSelPrio = false;
    private long timeout = -1;
    private VarVectors varVectors = new VarVectors();
    
    public long getTimeout(){
        return this.timeout;
    }
    
    public boolean isVarSelPrio(){
        return this.varSelPrio;
    }

    public HcaspBaseListener(Store store) {
        this.store = store;
    }

    public IntVar[] getOrderedVars(){  
        if(this.orderedVars.isEmpty()){
            return null;
        }
        
        return this.orderedVars.toArray(new IntVar[this.orderedVars.size()]);
    }
    
    public IntVar[] getVars() {
        Collection <IntVar> c = this.vars.values();
        if(c.isEmpty()){
            return null;
        }
        
        return c.toArray(new IntVar[c.size()]);
    }

    public ExitChildListener <IntVar> getExitChildListener(){
        return this.exitChildListener;
    }
    
    public Indomain <IntVar> getValSelect(){
        return this.valSelect;
    }
    
    public ComparatorVariable <IntVar> getVarSelect(){
        return this.varSelect;
    }
    
    public ComparatorVariable <IntVar> getVarSelectTieBreak(){
        return this.varSelectTieBreak;
    }
    
    @Override
    public void enterSearch(@NotNull HcaspParser.SearchContext ctx){
        if(ctx.CONST(0).getText().equals("lds")){
            int limit = Integer.parseInt(ctx.num(0).getText());
            if(this.exitChildListener == null){   
                this.exitChildListener = new MyLds();               
            }

            if(ctx.varset()==null){
                ((MyLds)exitChildListener).setDefault(limit);
                System.out.println("max discrepancies (default) = "+limit); 
            }
            else{
                IntVar[] vararr = this.varVectors.getVariableVector(ctx.varset());
                if(vararr!=null){
                    for(IntVar v:vararr){
                        ((MyLds)exitChildListener).setVariable(v, limit);
                    }
                }
                System.out.println("max discrepancies ("+ctx.varset().getText()+") = "+limit);
            }          
        }
        
        if(ctx.CONST(0).getText().equals("limited")){
            int limit = Integer.parseInt(ctx.num(0).getText());
            if(this.exitChildListener == null){   
                this.exitChildListener = new MyLimited<>();               
            }

            if(ctx.varset()==null){
                ((MyLimited)exitChildListener).setDefault(limit);
                System.out.println("max limit (default) = "+limit); 
            }
            else{
                IntVar[] vararr = this.varVectors.getVariableVector(ctx.varset());
                if(vararr!=null){
                    for(IntVar v:vararr){
                        ((MyLimited)exitChildListener).setVariable(v, limit);
                    }
                }
                System.out.println("max limit ("+ctx.varset().getText()+") = "+limit);
            }          
        }
    }
    
    private void insertOrdered(IntVar v, Integer prio){
        int i = 0;
        while(i < this.prios.size() && (this.prios.get(i) > prio)){
            i++;
        }
        this.prios.add(i, prio);
        this.orderedVars.add(i, v);
        //System.out.println(v.id+" has priority="+prio);
    }
    
    @Override
    public void enterVar(@NotNull HcaspParser.VarContext ctx){
        IntVar v = null;
        Integer prio = Integer.MIN_VALUE;
        
        if(ctx.num(2)!=null){
            prio = Integer.parseInt(ctx.num(2).getText());
            //System.out.println("prio found:"+prio);
        }
        
        if(ctx.varname().CONST()!=null){
            v = new IntVar(this.store,ctx.varname().CONST().getText(),Integer.parseInt(ctx.num(0).getText()),Integer.parseInt(ctx.num(1).getText()));
            this.vars.put(ctx.varname().CONST().getText(), v);
            //System.out.println("add variable: "+ctx.varname().getText());
            insertOrdered(v,prio);
        }
        else if(ctx.varname().varset()!=null){
            for(Iterator <HcaspParser.ArgContext> iter = ctx.varname().varset().arg().iterator(); iter.hasNext(); ){
                if(iter.next().all() != null){
                    throw new RuntimeException("'all' not allowed within variable declaration:"+ctx.getText());
                }
            }
            v = new IntVar(this.store,ctx.varname().varset().getText(),Integer.parseInt(ctx.num(0).getText()),Integer.parseInt(ctx.num(1).getText()));
            this.vars.put(ctx.varname().varset().getText(), v);
            //System.out.println("add variable: "+ctx.varname().getText());
            insertOrdered(v,prio);
            this.varVectors.addVariable(ctx.varname().varset(), v);
        }
    }
    
    @Override 
    public void enterConstr(@NotNull HcaspParser.ConstrContext ctx){ 
        //System.out.println("add constraint: "+ctx.getText());
        if(!ctx.CONST().getText().equals("eq") && 
                !ctx.CONST().getText().equals("neq") && 
                !ctx.CONST().getText().equals("lt") && 
                !ctx.CONST().getText().equals("lteq")&& 
                !ctx.CONST().getText().equals("gt") && 
                !ctx.CONST().getText().equals("gteq")){
            throw new RuntimeException("constraint not supported:"+ctx.getText());
        }
        else{
            IntVar a = this.vars.get(ctx.varname().getText());
            if(a == null){
                throw new RuntimeException("first variable not found:"+ctx.getText());
            }
            else{
                if(ctx.val().varname() != null){
                    IntVar b = this.vars.get(ctx.val().varname().getText());
                    if(b == null){
                        throw new RuntimeException("second variable not found:"+ctx.getText());
                    }
                    else if(ctx.CONST().getText().equals("eq")){
                        this.store.impose(new XeqY(a, b));
                    }
                    else if(ctx.CONST().getText().equals("neq")){
                        this.store.impose(new XneqY(a, b));
                    }
                    else if(ctx.CONST().getText().equals("lteq")){
                        this.store.impose(new XlteqY(a, b));
                    }
                    else if(ctx.CONST().getText().equals("lt")){
                        this.store.impose(new XltY(a, b));
                    }
                    else if(ctx.CONST().getText().equals("gteq")){
                        this.store.impose(new XgteqY(a, b));
                    }
                    else if(ctx.CONST().getText().equals("gt")){
                        this.store.impose(new XgtY(a, b));
                    }
                }
                else if(ctx.CONST().getText().equals("eq")){
                    this.store.impose(new XeqC(a, Integer.parseInt(ctx.val().num().getText())));
                }
                else if(ctx.CONST().getText().equals("neq")){
                    this.store.impose(new XneqC(a, Integer.parseInt(ctx.val().num().getText())));
                }
                else if(ctx.CONST().getText().equals("lteq")){
                    this.store.impose(new XlteqC(a, Integer.parseInt(ctx.val().num().getText())));
                }
                else if(ctx.CONST().getText().equals("lt")){
                    this.store.impose(new XltC(a, Integer.parseInt(ctx.val().num().getText())));
                }
                else if(ctx.CONST().getText().equals("gteq")){
                    this.store.impose(new XgteqC(a, Integer.parseInt(ctx.val().num().getText())));
                }
                else if(ctx.CONST().getText().equals("gt")){
                    this.store.impose(new XgtC(a, Integer.parseInt(ctx.val().num().getText())));
                }
            }
        }
    }
    
    @Override
    public void enterArith(@NotNull HcaspParser.ArithContext ctx){
        //System.out.println("add arith: "+ctx.getText());
        IntVar a = this.vars.get(ctx.varname(0).getText());
        IntVar b = this.vars.get(ctx.varname(1).getText());
        IntVar c = this.vars.get(ctx.val().varname().getText());
        if(a == null){
            throw new RuntimeException("first variable not found:"+ctx.getText());
        }
        else if(b == null){
            throw new RuntimeException("second variable not found:"+ctx.getText());            
        }
        else if(c == null){
            throw new RuntimeException("third variable not found:"+ctx.getText());            
        }
        else if(!ctx.CONST(1).getText().equals("eq")){
            throw new RuntimeException("constraint not supported:"+ctx.getText());
        }
        else if(ctx.CONST(0).getText().equals("plus")){
            this.store.impose(new XplusYeqZ(a,b,c));
        }
        else if(ctx.CONST(0).getText().equals("mult")){
            this.store.impose(new XmulYeqZ(a,b,c));
        }
        else if(ctx.CONST(0).getText().equals("exp")){
            this.store.impose(new XexpYeqZ(a,b,c));
        }
        else if(ctx.CONST(0).getText().equals("minus")){
            this.store.impose(new XplusYeqZ(b,c,a));
        }
        else if(ctx.CONST(0).getText().equals("div")){
            this.store.impose(new XmulYeqZ(b,c,a));
        }
        else{
            throw new RuntimeException("constraint not supported:"+ctx.getText());
        }
    }

    @Override
    public void enterGlobal(@NotNull HcaspParser.GlobalContext ctx){
        //System.out.println("add global: "+ctx.getText());
        if(ctx.CONST().getText().equals("alldiff")){
            IntVar[] v = this.varVectors.getVariableVector(ctx.varset(0));
            if(v != null){                
                this.store.impose(new Alldiff(v));
            }
        }
        else if(ctx.CONST().getText().equals("sum")){
            IntVar[] v = this.varVectors.getVariableVector(ctx.varset(0));
            IntVar s = this.vars.get(ctx.val(0).varname().getText());
            if(v != null){
                this.store.impose(new Sum(v,s));
            }
        }
        else if(ctx.CONST().getText().equals("max")){
            IntVar[] v = this.varVectors.getVariableVector(ctx.varset(0));
            IntVar s = this.vars.get(ctx.val(0).varname().getText());
            if(v != null){
                // System.out.println("global max");
                this.store.impose(new Max(v,s));
            }
        }
        else if(ctx.CONST().getText().equals("min")){
            IntVar[] v = this.varVectors.getVariableVector(ctx.varset(0));
            IntVar s = this.vars.get(ctx.val(0).varname().getText());
            if(v != null){
                this.store.impose(new Min(v,s));
            }
        }
        else if(ctx.CONST().getText().equals("gcc")){
            //System.out.println(ctx.getText());
            IntVar[] v = this.varVectors.getVariableVector(ctx.varset(0));
            IntVar[] c = this.varVectors.getVariableVector(ctx.varset(1));
            //System.out.println("gcc: num vars:"+v.length+" num counters:"+c.length);
            if(v != null){
                this.store.impose(new GCC(v,c));
                //this.store.impose(new MyGcc(v,c));
            }
        }
        else if(ctx.CONST().getText().equals("count")){
            //System.out.println(ctx.getText());
            IntVar[] v = this.varVectors.getVariableVector(ctx.varset(0));
            IntVar c = this.vars.get(ctx.val(0).varname().getText());
            int i = Integer.parseInt(ctx.val(1).num().getText());
            
            if(v != null){
                this.store.impose(new Count(v,c,i));
            }
            else{
                System.err.println("varVector is null: "+ctx.getText());
            }
        }
        else if(ctx.CONST().getText().equals("cumulative")){
            IntVar[] a = this.varVectors.getVariableVector(ctx.varset(0));
            IntVar[] b = this.varVectors.getVariableVector(ctx.varset(1));
            IntVar[] c = this.varVectors.getVariableVector(ctx.varset(2));
            IntVar d = this.vars.get(ctx.val(0).varname().getText());
            
            if(a == null){
                throw new RuntimeException("cumulative: first variable vector not found");
            }
            else if(b == null){
                throw new RuntimeException("cumulative: second variable vector not found");
            }
            else if(c == null){
                throw new RuntimeException("cumulative: third variable vector not found");
            }
            else if(d == null){
                throw new RuntimeException("cumulative: variable not found"+ctx.val(0).varname().getText());
            }
            else{
                this.store.impose(new Cumulative(a, b, c, d));
            }
        }
        else if(ctx.CONST().getText().equals("element")){
            //System.out.println("get data for element constraint");
            IntVar index = this.vars.get(ctx.val(0).varname().getText());
            IntVar[] vec = this.varVectors.getVariableVector(ctx.varset(0));
            IntVar value = this.vars.get(ctx.val(1).varname().getText());
            
            if(index == null){
                throw new RuntimeException("element: index variable not found");
            }
            else if(vec == null){
                throw new RuntimeException("element: variable vector not found");
            }
            else if(index == null){
                throw new RuntimeException("element: value variable not found");
            }
            else{
                /*String str = "impose element constraint: "+index.id+" in |";
                for(IntVar v:vec){
                    str+=v.id+"|";
                }
                System.out.println(str+" = "+value.id);*/
                //System.out.println("imposing element constraint");
                this.store.impose(new Element(index, vec, value));
                //System.out.println("imposing element constraint");
            }
        }
        else if(ctx.CONST().getText().equals("binpacking")){
            IntVar[] a = this.varVectors.getVariableVector(ctx.varset(0));
            IntVar[] b = this.varVectors.getVariableVector(ctx.varset(1));
            IntVar[] c = this.varVectors.getVariableVector(ctx.varset(2));
            
            if(a == null){
                throw new RuntimeException("binpacking: first variable vector not found");
            }
            else if(b == null){
                throw new RuntimeException("binpacking: second variable vector not found");
            }
            else if(c == null){
                throw new RuntimeException("binpacking: third variable vector not found");
            }
            else{
                int[] size = new int[c.length];
                for(int i = 0; i < size.length; i++){
                    size[i] = c[i].min();
                }
                this.store.impose(new Binpacking(a, b, size));
            }
        }
        else{
            throw new RuntimeException("constraint not supported:"+ctx.getText()); 
        }
    }
    
    @Override
    public void enterValsel(@NotNull HcaspParser.ValselContext ctx) {
        String h = ctx.CONST(0).getText();
        if(ctx.varset()==null){
            if(h.equals("indomainMin")||h.equals("indomainMiddle")||h.equals("indomainMax")||h.equals("indomainRandom")){
                System.out.println(h);
                this.valSelect.addDefault(h);
            }
            else if(h.equals("indomainPreferred")){
                String h1 = "indomainMin";
                if(ctx.CONST(1)!=null){
                    h1 = ctx.CONST(1).getText();
                }
                System.out.println(h+"-"+h1);
                this.valSelect.addDefault(h,h1);
            }
            else{
                throw new RuntimeException("heuristic not supported:"+ctx.CONST(0).getText());
            }
        }
        else{//varset!=null
            IntVar[] vararr = this.varVectors.getVariableVector(ctx.varset());

            if(vararr!=null){
                for(IntVar v:vararr){
                    if(h.equals("indomainMin")||h.equals("indomainMiddle")||h.equals("indomainMax")||h.equals("indomainRandom")){
                        //System.out.println(v.id+":"+h);
                        this.valSelect.addVariable(v,h);
                    }
                    else if(h.equals("indomainPreferred")){
                        String h1 = "indomainMin";
                        if(ctx.CONST(1)!=null){
                            h1 = ctx.CONST(1).getText();
                        }
                        //System.out.println(v.id+":"+h+"-"+h1);
                        this.valSelect.addVariable(v,h,h1);
                    }
                    else{
                        throw new RuntimeException("heuristic not supported:"+ctx.CONST(0).getText());
                    }
                }
            }
        }
    }
    
    @Override
    public void enterVarsel(@NotNull HcaspParser.VarselContext ctx) {
        if(ctx.CONST(0).getText().equals("smallestDomain")){
            System.out.println("varsel:smallestDomain");
            this.varSelect = new SmallestDomain<>();
        }
        else if(ctx.CONST(0).getText().equals("largestDomain")){
            System.out.println("varsel:largestDomain");
            this.varSelect = new LargestDomain<>();
        }
        else if(ctx.CONST(0).getText().equals("smallestMin")){
            System.out.println("varsel:smallestMin");
            this.varSelect = new SmallestMin<>();
        }
        else if(ctx.CONST(0).getText().equals("smallestMax")){
            System.out.println("varsel:smallestMax");
            this.varSelect = new SmallestMax<>();
        }
        else if(ctx.CONST(0).getText().equals("mostConstrainedStatic")){
            System.out.println("varsel:mostConstrainedStatic");
            this.varSelect = new MostConstrainedStatic<>();
        }
        else if(ctx.CONST(0).getText().equals("mostConstrainedDynamic")){
            System.out.println("varsel:mostConstrainedDynamic");
            this.varSelect = new MostConstrainedDynamic<>();
        }
        else if(ctx.CONST(0).getText().equals("maxRegret")){
            System.out.println("varsel:maxRegret");
            this.varSelect = new MaxRegret<>();
        }
        else if(ctx.CONST(0).getText().equals("weightedDegree")){
            System.out.println("varsel:weightedDegree");
            this.varSelect = new WeightedDegree<>();
        }
        else if(ctx.CONST(0).getText().equals("priority")){
            System.out.println("varsel:priority");
            this.varSelPrio = true;
        }
        else{
            throw new RuntimeException("heuristic not supported:"+ctx.CONST(0).getText());
        }
        
        if(ctx.CONST(1)!=null){
            if(ctx.CONST(1).getText().equals("smallestDomain")){
                System.out.println("tiebreak:smallestDomain");
                this.varSelectTieBreak = new SmallestDomain<>();
            }
            else if(ctx.CONST(1).getText().equals("largestDomain")){
                System.out.println("tiebreak:largestDomain");
                this.varSelectTieBreak = new LargestDomain<>();
            }
            else if(ctx.CONST(1).getText().equals("smallestMin")){
                System.out.println("tiebreak:smallestMin");
                this.varSelectTieBreak = new SmallestMin<>();
            }
            else if(ctx.CONST(1).getText().equals("smallestMax")){
                System.out.println("tiebreak:smallestMax");
                this.varSelectTieBreak = new SmallestMax<>();
            }
            else if(ctx.CONST(1).getText().equals("mostConstrainedStatic")){
                System.out.println("tiebreak:mostConstrainedStatic");
                this.varSelectTieBreak = new MostConstrainedStatic<>();
            }
            else if(ctx.CONST(1).getText().equals("mostConstrainedDynamic")){
                System.out.println("tiebreak:mostConstrainedDynamic");
                this.varSelectTieBreak = new MostConstrainedDynamic<>();
            }
            else if(ctx.CONST(1).getText().equals("maxRegret")){
                System.out.println("tiebreak:maxRegret");
                this.varSelectTieBreak = new MaxRegret<>();
            }
            else if(ctx.CONST(1).getText().equals("weightedDegree")){
                System.out.println("tiebreak:weightedDegree");
                this.varSelectTieBreak = new WeightedDegree<>();
            }
            else{
                throw new RuntimeException("heuristic not supported:"+ctx.CONST(1).getText());
            }
        }
    }

    @Override public void enterPrefer(@NotNull HcaspParser.PreferContext ctx){
        IntVar var = this.vars.get(ctx.varname(0).getText());
        IntVar val = this.vars.get(ctx.varname(1).getText());
        int prio = Integer.MIN_VALUE;
        
        if(var == null || val == null){
            throw new RuntimeException("variable not found:"+ctx.getText());
        }
        
        if(ctx.num()!=null){
            prio = Integer.parseInt(ctx.num().getText());
        }

        String rel = ctx.CONST().getText();
        
        if(rel.equals("eq")||rel.equals("neq")||rel.equals("lteq")||rel.equals("lt")||rel.equals("gteq")||rel.equals("gt")){
            this.valSelect.addPrefVal(var, rel, val, prio);
        }                                                                                                                                                             
        else{
            throw new RuntimeException("relation not supported:"+ctx.getText());
        }
    }
	
    private PrimitiveConstraint buildConstraint(HcaspParser.AnteContext ctx){
        String op = ctx.CONST().getText();
        
        if(!op.equals("eq") && !op.equals("neq") && !op.equals("lt") &&!op.equals("lteq")&& !op.equals("gt") && !op.equals("gteq")){
            throw new RuntimeException("constraint not supported:"+ctx.getText());
        }
        
        IntVar a = this.vars.get(ctx.varname().getText());
        
        if(a == null){
            throw new RuntimeException("first variable not found:"+ctx.getText());
        }
        
        PrimitiveConstraint c1;
        
        if(ctx.val().num() != null){
          int n = Integer.parseInt(ctx.val().num().getText());
          
          if(op.equals("neq")){
            c1=new XneqC(a, n);
            }
            else if(op.equals("lteq")){
                c1=new XlteqC(a, n);
            }
            else if(op.equals("lt")){
                c1=new XltC(a, n);
            }
            else if(op.equals("gteq")){
                c1=new XgteqC(a, n);
            }
            else if(op.equals("gt")){
                c1=new XgtC(a, n);
            }
            else{
                c1=new XeqC(a, n);
            }
        }
        else{
            IntVar b = this.vars.get(ctx.val().varname().getText());
        
            if(b == null){
                throw new RuntimeException("second variable not found:"+ctx.getText());            
            }
            
            if(op.equals("neq")){
                c1=new XneqY(a, b);
            }
            else if(op.equals("lteq")){
                c1=new XlteqY(a, b);
            }
            else if(op.equals("lt")){
                c1=new XltY(a, b);
            }
            else if(op.equals("gteq")){
                c1=new XgteqY(a, b);
            }
            else if(op.equals("gt")){
                c1=new XgtY(a, b);
            }
            else{
                c1=new XeqY(a, b);
            }
        }
        
        return c1;
    }
    
    private PrimitiveConstraint buildConstraint(HcaspParser.SuccContext ctx){
        String op = ctx.CONST().getText();
        
        if(!op.equals("eq") && !op.equals("neq") && !op.equals("lt") &&!op.equals("lteq")&& !op.equals("gt") && !op.equals("gteq")){
            throw new RuntimeException("constraint not supported:"+ctx.getText());
        }
        
        IntVar a = this.vars.get(ctx.varname().getText());
        
        if(a == null){
            throw new RuntimeException("first variable not found:"+ctx.getText());
        }
        
        PrimitiveConstraint c1;
        
        if(ctx.val().num() != null){
          int n = Integer.parseInt(ctx.val().num().getText());
          
          if(op.equals("neq")){
            c1=new XneqC(a, n);
            }
            else if(op.equals("lteq")){
                c1=new XlteqC(a, n);
            }
            else if(op.equals("lt")){
                c1=new XltC(a, n);
            }
            else if(op.equals("gteq")){
                c1=new XgteqC(a, n);
            }
            else if(op.equals("gt")){
                c1=new XgtC(a, n);
            }
            else{
                c1=new XeqC(a, n);
            }
        }
        else{
            IntVar b = this.vars.get(ctx.val().varname().getText());
        
            if(b == null){
                throw new RuntimeException("second variable not found:"+ctx.getText());            
            }
            
            if(op.equals("neq")){
                c1=new XneqY(a, b);
            }
            else if(op.equals("lteq")){
                c1=new XlteqY(a, b);
            }
            else if(op.equals("lt")){
                c1=new XltY(a, b);
            }
            else if(op.equals("gteq")){
                c1=new XgteqY(a, b);
            }
            else if(op.equals("gt")){
                c1=new XgtY(a, b);
            }
            else{
                c1=new XeqY(a, b);
            }
        }
        
        return c1;
    }
    
    @Override public void enterWenn(@NotNull HcaspParser.WennContext ctx) {      
        List <HcaspParser.AnteContext> anteCons = ctx.ante();
        
        PrimitiveConstraint[] ante = new PrimitiveConstraint[anteCons.size()];
        for(int i = 0; i < anteCons.size(); i++){
            ante[i] = buildConstraint(anteCons.get(i));
        }

        And and = new And(ante);
        
        List <HcaspParser.SuccContext> succCons = ctx.succ();
        
        PrimitiveConstraint[] succ = new PrimitiveConstraint[succCons.size()];
        for(int i = 0; i < succCons.size(); i++){
            succ[i] = buildConstraint(succCons.get(i));
        }

        Or or = new Or(succ);

        this.store.impose(new IfThen(and, or));
    }    
    
    @Override public void enterTimeout(@NotNull HcaspParser.TimeoutContext ctx) { 
        this.timeout = Long.parseLong(ctx.num().getText());
        System.out.println("csp timeout = " + this.timeout);
    }
    
//not needed!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
    @Override public void exitWenn(@NotNull HcaspParser.WennContext ctx) { }
    @Override public void enterVal(@NotNull HcaspParser.ValContext ctx) { }
    @Override public void exitVal(@NotNull HcaspParser.ValContext ctx) { }
    @Override public void exitArith(@NotNull HcaspParser.ArithContext ctx) { }
    @Override public void exitVar(@NotNull HcaspParser.VarContext ctx) { }
    @Override public void exitGlobal(@NotNull HcaspParser.GlobalContext ctx) { }
    @Override public void enterVarset(@NotNull HcaspParser.VarsetContext ctx) { }
    @Override public void exitVarset(@NotNull HcaspParser.VarsetContext ctx) { }
    @Override public void enterNum(@NotNull HcaspParser.NumContext ctx) { }
    @Override public void exitNum(@NotNull HcaspParser.NumContext ctx) { }
    @Override public void exitValsel(@NotNull HcaspParser.ValselContext ctx) { }
    @Override public void exitSearch(@NotNull HcaspParser.SearchContext ctx) { }
    @Override public void exitConstr(@NotNull HcaspParser.ConstrContext ctx) { }
    @Override public void exitVarsel(@NotNull HcaspParser.VarselContext ctx) { }
    @Override public void enterVarname(@NotNull HcaspParser.VarnameContext ctx) { }
    @Override public void exitVarname(@NotNull HcaspParser.VarnameContext ctx) { }
    @Override public void enterCsp(@NotNull HcaspParser.CspContext ctx) { }
    @Override public void exitCsp(@NotNull HcaspParser.CspContext ctx) { }
    @Override public void enterEveryRule(@NotNull ParserRuleContext ctx) { }
    @Override public void exitEveryRule(@NotNull ParserRuleContext ctx) { }
    @Override public void visitTerminal(@NotNull TerminalNode node) { }
    @Override public void visitErrorNode(@NotNull ErrorNode node) { }
    @Override public void enterArg(@NotNull HcaspParser.ArgContext ctx) { }
    @Override public void exitArg(@NotNull HcaspParser.ArgContext ctx) { }
    @Override public void exitPrefer(@NotNull HcaspParser.PreferContext ctx){};
    @Override public void enterAll(@NotNull HcaspParser.AllContext ctx) { }
    @Override public void exitAll(@NotNull HcaspParser.AllContext ctx) { }
    @Override public void exitSucc(@NotNull HcaspParser.SuccContext ctx) { }
    @Override public void exitAnte(@NotNull HcaspParser.AnteContext ctx) { }
    @Override public void enterSucc(@NotNull HcaspParser.SuccContext ctx) { }
    @Override public void enterAnte(@NotNull HcaspParser.AnteContext ctx) { }
    @Override public void exitTimeout(@NotNull HcaspParser.TimeoutContext ctx) { }
}