/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ascass.solving;

import java.util.ArrayList;
import java.util.Iterator;
import org.jacop.core.IntVar;
import org.jacop.core.Store;
import org.jacop.search.ComparatorVariable;
import org.jacop.search.DepthFirstSearch;
import org.jacop.search.Indomain;
import org.jacop.search.InputOrderSelect;
import org.jacop.search.MostConstrainedDynamic;
import org.jacop.search.Search;
import org.jacop.search.SelectChoicePoint;
import org.jacop.search.SimpleSelect;
import org.jacop.search.SmallestDomain;
import org.antlr.v4.runtime.*;
import org.antlr.v4.runtime.tree.*;
import org.jacop.search.IndomainMin;
import org.jacop.search.MostConstrainedStatic;
import ascass.parsing.HcaspBaseListener;
import ascass.parsing.HcaspLexer;
import ascass.parsing.HcaspParser;

/**
 *
 * @author erich
 */
public class CaspSolver {
    private final AspSolver aspSolver;
    private final String source;
    private Store store;
    public int aspTime;
    public int cspTime;
    public int createCspTime;
    public boolean complete;
    private String atoms = "";
        
    public Store getStore() {
		return store;
	}

	public CaspSolver(String source) {        
        this.source = source;
        this.aspSolver = new AspSolver(this.source);
    }

    public AspSolver getAspSolver() {
        return this.aspSolver;
    }
    
    public String getSource() {
        return this.source;
    }
        
    private ArrayList<ComparatorVariable> getVarSelectors(){
        ArrayList <ComparatorVariable> list = new ArrayList();
        
        list.add(new SmallestDomain());
        list.add(new MostConstrainedStatic());
        list.add(new org.jacop.search.MostConstrainedDynamic());
        list.add(new org.jacop.search.SmallestMax<>());
        list.add(new org.jacop.search.SmallestMin<>());
        list.add(new org.jacop.search.WeightedDegree<>());
        list.add(new org.jacop.search.LargestDomain<>());
        list.add(new org.jacop.search.LargestMax<>());
        list.add(new org.jacop.search.LargestMin<>());
        list.add(new org.jacop.search.MaxRegret<>());
        list.add(new org.jacop.search.MinDomainOverDegree<>());
        //System.out.println("builin heuristic list size = "+list.size());
        return list;
    }
    
    public String simpleHeuristic()throws Exception{
        int timelimit = 16;
        int ldslimit = 4;
        int breadthlimit = 4;
        
        boolean solved = this.aspSolver.solve();
         if(solved){
            this.aspSolver.nextAnswerSet();
            

            ANTLRInputStream input = new ANTLRInputStream(this.aspSolver.getCsp());
            HcaspLexer lexer = new HcaspLexer(input);
            CommonTokenStream tokens = new CommonTokenStream(lexer);
            HcaspParser parser = new HcaspParser(tokens);
            ParseTree tree = parser.csp();

            ParseTreeWalker walker = new ParseTreeWalker();
            
            

            String heuristics = "";
            
            for(int t = 1; t <= timelimit; t*=2){
                for(int l = 1; l <= ldslimit; l*=2){
                    Search<IntVar> search = new DepthFirstSearch<>();
                    search.setTimeOut(t);
                    MyLds lds = new MyLds();
                    lds.setDefault(ldslimit);
                    search.setExitChildListener(lds);
                    
                    ComparatorVariable selector;
                    ComparatorVariable tiebreaker;
                    
                    ArrayList<ComparatorVariable> selectors = getVarSelectors();
                    ArrayList<ComparatorVariable> tiebreakers = getVarSelectors();
                    
                    for(int i = 0; i < selectors.size(); i++){
                        selector = selectors.get(i);
                        for(int j = 0; j < tiebreakers.size(); j++){
                            Store s = new Store();
                            HcaspBaseListener listener = new HcaspBaseListener(s);
                            walker.walk(listener, tree);

                            if(listener.getVars() == null)
                                return "no csp variables";
                        
                            tiebreaker = tiebreakers.get(j);
                            SelectChoicePoint<IntVar> select;
                            select = new SimpleSelect<>(listener.getVars(),selector,tiebreaker,new IndomainMin<IntVar>());
                            System.out.println("time limit = " + t+", lds limit = " + l+", var selector = " + selector.getClass().getName()+", tiebreaker = " + tiebreaker.getClass().getName());
                            if(search.labeling(s, select)){
                                heuristics += "time limit = " + t+", lds limit = " + l+", var selector = " + selector.getClass().getName()+", tiebreaker = " + tiebreaker.getClass().getName()+ "\n";
                            }
                        }
                    }
                }
                for(int l = 1; l <= breadthlimit; l*=2){
                    Search<IntVar> search = new DepthFirstSearch<>();
                    search.setTimeOut(t);
                    MyLimited limited = new MyLimited();
                    limited.setDefault(ldslimit);
                    search.setExitChildListener(limited);
                    
                    ComparatorVariable selector;
                    ComparatorVariable tiebreaker;
                    
                    ArrayList<ComparatorVariable> selectors = getVarSelectors();
                    ArrayList<ComparatorVariable> tiebreakers = getVarSelectors();
                    
                    for(int i = 0; i < selectors.size(); i++){
                        selector = selectors.get(i);
                        for(int j = 0; j < tiebreakers.size(); j++){
                            Store s = new Store();
                            HcaspBaseListener listener = new HcaspBaseListener(s);
                            walker.walk(listener, tree);

                            if(listener.getVars() == null)
                                return "no csp variables";
                            
                            tiebreaker = tiebreakers.get(j);
                            SelectChoicePoint<IntVar> select;
                            select = new SimpleSelect<>(listener.getVars(),selector,tiebreaker,new IndomainMin<IntVar>());
                            System.out.println("time limit = " + t+", breadth limit = " + l+", var selector = " + selector.getClass().getName()+", tiebreaker = " + tiebreaker.getClass().getName());
                            if(search.labeling(s, select)){
                                heuristics += "time limit = " + t+", breadth limit = " + l+", var selector = " + selector.getClass().getName()+", tiebreaker = " + tiebreaker.getClass().getName()+ "\n";
                            }
                        }
                    }
                }
                {//unlimited
                    Search<IntVar> search = new DepthFirstSearch<>();
                    search.setTimeOut(t);
                    
                    ComparatorVariable selector;
                    ComparatorVariable tiebreaker;
                    
                    ArrayList<ComparatorVariable> selectors = getVarSelectors();
                    ArrayList<ComparatorVariable> tiebreakers = getVarSelectors();
                    
                    for(int i = 0; i < selectors.size(); i++){
                        selector = selectors.get(i);
                        for(int j = 0; j < tiebreakers.size(); j++){
                            Store s = new Store();
                            HcaspBaseListener listener = new HcaspBaseListener(s);
                            walker.walk(listener, tree);

                            if(listener.getVars() == null)
                                return "no csp variables";
                            
                            tiebreaker = tiebreakers.get(j);
                            SelectChoicePoint<IntVar> select;
                            select = new SimpleSelect<>(listener.getVars(),selector,tiebreaker,new IndomainMin<IntVar>());
                            System.out.println("time limit = " + t+", var selector = " + selector.getClass().getName()+", tiebreaker = " + tiebreaker.getClass().getName());
                            if(search.labeling(s, select)){
                                heuristics += "time limit = " + t+", var selector = " + selector.getClass().getName()+", tiebreaker = " + tiebreaker.getClass().getName()+ "\n";
                            }
                        }
                    }
                }
                
                if(!heuristics.isEmpty()){
                    return heuristics;
                }
            }
            return " could not solve instance";
        }
        else{
            return "no answer set found";
        }
    }
    
    public int calcSolution()throws Exception{
        this.complete = true;
        
        System.out.println("search answer sets");
        
        long startAsp;
        long endAsp;
        long startCsp;
        long endCsp;
        
        startAsp = System.currentTimeMillis();
        boolean solved = this.aspSolver.solve();
        endAsp = System.currentTimeMillis();
        this.aspTime = (int)(endAsp-startAsp)/1000;
        System.out.println("asp time (secs):"+this.aspTime);
        startCsp = System.currentTimeMillis();
        if(solved){ 
            System.out.println("found answer sets");
            while(this.aspSolver.nextAnswerSet()){
                System.out.println("process answer set");
                if(calcCspSolution()){
                    endCsp = System.currentTimeMillis();
                    this.cspTime = (int)(endCsp-startCsp)/1000;
                    System.out.println("csp time (secs):"+this.cspTime);
                    System.out.println("satisfiable");
                    return 1;
                }
            }
        }
        else if(this.aspSolver.timeout > -1 && this.aspTime >= this.aspSolver.timeout){
            this.cspTime = -1;
            System.out.println("csp time (secs):"+this.cspTime);
            System.out.println("undecided");
            return -2;
        }
        else{
            this.cspTime = -1;
            System.out.println("csp time (secs):"+this.cspTime);
            System.out.println("unsatisfiable");
            return 0;
        }
        
        endCsp = System.currentTimeMillis();
        this.cspTime = (int)(endCsp-startCsp)/1000;
        System.out.println("csp time (secs):"+this.cspTime);

        if(this.aspSolver.complete && this.complete){
            System.out.println("unsatisfiable");
            return 0;
        }
        else if(this.aspSolver.complete){
            System.out.println("no solution found");
            return -1;
        }
        else{
            System.out.println("no solution found");
            return -2;
        }
    }  
    
    private boolean calcCspSolution() throws Exception{
        System.out.println("instantiate csp");
        long startCreate;
        long endCreate;
        startCreate = System.currentTimeMillis();
        this.store = new Store();

        ANTLRInputStream input = new ANTLRInputStream(this.aspSolver.getCsp());
        HcaspLexer lexer = new HcaspLexer(input);
        CommonTokenStream tokens = new CommonTokenStream(lexer);
        HcaspParser parser = new HcaspParser(tokens);
        ParseTree tree = parser.csp();

        System.out.println("parse tree created");
        
        ParseTreeWalker walker = new ParseTreeWalker();
        HcaspBaseListener listener = new HcaspBaseListener(this.store);
        walker.walk(listener, tree);

        System.out.println("walked parse tree");
        
        if(listener.getVars() == null)
            return true;
        
        Search<IntVar> search = new DepthFirstSearch<>();
        if(listener.getExitChildListener()!=null){
            this.complete = false;
            search.setExitChildListener(listener.getExitChildListener());
        }
        
        if(listener.getTimeout()>=0){
            search.setTimeOut(listener.getTimeout());
        }
        
        Indomain<IntVar> valSelect = listener.getValSelect();
        ComparatorVariable<IntVar> varSelect = new SmallestDomain <>();
        ComparatorVariable<IntVar> varSelectTieBreak = new MostConstrainedDynamic<>();
        
        if(listener.getVarSelect() != null){
            varSelect = listener.getVarSelect();
        }
        
        if(listener.getVarSelectTieBreak() != null){
            varSelectTieBreak = listener.getVarSelectTieBreak();
        }
        
        SelectChoicePoint<IntVar> select;
        
        if(listener.isVarSelPrio()){
            select = new InputOrderSelect(store, listener.getOrderedVars(), valSelect);
            /*for(IntVar v : listener.getOrderedVars()){
                System.out.println(v.id);
            }*/
        }
        else{
            select = new SimpleSelect<>(listener.getVars(),varSelect,varSelectTieBreak,valSelect);
        }     
        
        endCreate = System.currentTimeMillis();
        this.createCspTime = (int)(endCreate-startCreate)/1000;
        System.out.println("csp create time (secs):"+this.createCspTime);
        
        System.out.println("begin search");
        boolean solved;
        long startSearch = System.currentTimeMillis();
        solved = search.labeling(store, select);
        int timeSearch = (int)(System.currentTimeMillis()-startSearch)/1000;
        
        if(listener.getTimeout() >= 0 && timeSearch >= listener.getTimeout()){
            this.complete = false;
        }
        
        StringBuffer strBuf = new StringBuffer();
        
        if(solved){
            IntVar[] arr = listener.getOrderedVars();
            IntVar v;
            
            for(int i = 0; i < arr.length; i++){
                v = arr[i];
                strBuf.append(getAtom(v));
                strBuf.append(".\n");
            }
            
            this.atoms = strBuf.toString();
        }
        
        return solved;
    }
    
    public String getAtoms(){
        return this.atoms;
    }
    
    private String getAtom(IntVar v){
        String name = v.id;
        
        if(name.endsWith(")")){
            return name.replace(')', ',')+v.value()+")";
        }
        else{
            return name+"("+v.value()+")";
        }
    }
    
    /*
*/
    
    public String getCspSolution(){
        String str = "";
        for(int i = 0; i < this.store.vars.length; i++){
            if(this.store.vars[i]!=null){
                str+=this.store.vars[i]+"\n";
            }
        }
        return str;
    }
    
}
