/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ascass.solving;

import org.jacop.core.IntVar;
import org.jacop.search.Indomain;
import org.jacop.search.IndomainMin;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 *
 * @author erich
 */
public class IndomainPreferred implements Indomain<IntVar> {
    private final Indomain<IntVar> defIndomain;
    private final HashMap<IntVar,ArrayList<PrefVal>> prefVals = new HashMap<>();
    
    public IndomainPreferred(Indomain<IntVar> defIndomain) {
        if(defIndomain!=null){
            this.defIndomain = defIndomain;
        }
        else{
            this.defIndomain = new IndomainMin<>();
        }
    }
    
    public void addPrefVal(IntVar var, String rel, IntVar val){
        addPrefVal(var,rel,val,Integer.MIN_VALUE);
    }
    
    public void addPrefVal(IntVar var, String rel, IntVar val, int prio){
        if(!this.prefVals.containsKey(var)){
            this.prefVals.put(var, new ArrayList<PrefVal>());
        }
        
        ArrayList<PrefVal> vals = this.prefVals.get(var);
        int pos = 0;
        while(pos < vals.size() && vals.get(pos).prio > prio){
            pos++;
        }
        
        vals.add(pos,new PrefVal(rel,val,prio));
    }
    
    @Override public int indomain(IntVar var) throws RuntimeException {
        ArrayList<PrefVal> vals = this.prefVals.get(var);
        PrefVal prefVal;
        //System.out.println("select on: "+var.id);
        int v;
        
        if(vals!=null){
            //System.out.println(var.id+": preferred values given");
            
            for(Iterator<PrefVal> iter = vals.iterator(); iter.hasNext();){
                prefVal = iter.next();
                
                if(prefVal.val.dom().singleton()){
                    v = prefVal.val.value();
                }
                else{
                    v = prefVal.val.min();
                }                

                if(prefVal.rel.equals("neq")){
                    for(int i = var.domain.min(); i <= var.domain.max(); i++){
                        if(var.domain.contains(i) && i != v){
                            //System.out.println("domVal "+i+" neq prefVal "+v);
                            return i;
                        }
                    }
                }
                else if(prefVal.rel.equals("lteq")){
                    for(int i = v; i >= var.domain.min(); i--){
                        if(var.domain.contains(i) && i <= v){
                            //System.out.println("domVal "+i+" lteq prefVal "+v);
                            return i;
                        }
                    }
                }
                else if(prefVal.rel.equals("lt")){
                    for(int i = v-1; i >= var.domain.min(); i--){
                        if(var.domain.contains(i) && i < v){
                            //System.out.println("domVal "+i+" lt prefVal "+v);
                            return i;
                        }
                    }
                }
                else if(prefVal.rel.equals("gteq")){
                    for(int i = v; i <= var.domain.max(); i++){
                        if(var.domain.contains(i)){
                            //System.out.println("domVal "+i+" gteq prefVal "+v);
                            return i;
                        }
                    }
                }
                else if(prefVal.rel.equals("gt")){
                    for(int i = v+1; i <= var.domain.max(); i++){
                        if(var.domain.contains(i)){
                            //System.out.println("domVal "+i+" gt prefVal "+v);
                            return i;
                        }
                    }
                }
                else{
                    if(var.domain.contains(v)){
                        //System.out.println("domVal eq prefVal "+v);
                            return v;
                    }
                }                
            }
        }
        //System.out.println("use default strategy:"+defIndomain.getClass().getName());
        return defIndomain.indomain(var);
    }


    private static class PrefVal{
        String rel;
        IntVar val;
        Integer prio;
        PrefVal(String rel, IntVar val, int prio){
            this.rel = rel;
            this.val = val;
            this.prio = prio;
        }
    }

}