package ascass.solving;

import org.jacop.core.IntVar;
import java.util.HashMap;
import org.jacop.search.Indomain;
import org.jacop.search.IndomainMax;
import org.jacop.search.IndomainMiddle;
import org.jacop.search.IndomainMin;
import org.jacop.search.IndomainRandom;

public class MyIndomainHierarchical implements Indomain<IntVar> {
    private final HashMap<IntVar, Indomain<IntVar>> map = new HashMap<>();
    private final IndomainMin<IntVar> indomainMin = new IndomainMin<>();
    private final IndomainMiddle<IntVar> indomainMiddle = new IndomainMiddle<>();
    private final IndomainMax<IntVar> indomainMax = new IndomainMax<>();
    private final IndomainRandom<IntVar> indomainRandom = new IndomainRandom<>();
    private final IndomainPreferred indomainPreferredMin = new IndomainPreferred(indomainMin);
    private final IndomainPreferred indomainPreferredMid = new IndomainPreferred(indomainMiddle);
    private final IndomainPreferred indomainPreferredMax = new IndomainPreferred(indomainMax);
    private final IndomainPreferred indomainPreferredRnd = new IndomainPreferred(indomainRandom);
    private Indomain<IntVar> indomainDef = indomainMin;
    
    
    public MyIndomainHierarchical() {
    }
    
    public void addPrefVal(IntVar var, String rel, IntVar val){
        addPrefVal(var,rel,val,Integer.MIN_VALUE);
    }
    
    public void addPrefVal(IntVar var, String rel, IntVar val, int prio){
        Indomain<IntVar> sel = this.map.get(var);
        
        if(sel != null && sel instanceof IndomainPreferred){
            ((IndomainPreferred)sel).addPrefVal(var, rel, val, prio);
        }
        else if(indomainDef instanceof IndomainPreferred){
            ((IndomainPreferred)indomainDef).addPrefVal(var, rel, val, prio);
        }
    }
    
    public void addDefault(String h){
        if(h.equals("indomainMin")){
            indomainDef = indomainMin;
        }
        else if(h.equals("indomainMiddle")){
            indomainDef = indomainMiddle;
        }
        else if(h.equals("indomainMax")){
           indomainDef = indomainMax;
        }
        else if(h.equals("indomainRandom")){
            indomainDef = indomainRandom;
        }
        else if(h.equals("indomainPreferred")){
            indomainDef = indomainPreferredMin;
        }

    }
    
    public void addDefault(String p, String h){
        if(p.equals("indomainPreferred")){
            if(h.equals("indomainMiddle")){
                indomainDef = indomainPreferredMid;
            }
            else if(h.equals("indomainMax")){
                indomainDef = indomainPreferredMax;
            }
            else if(h.equals("indomainRandom")){
                indomainDef = indomainPreferredRnd;
            }
            else{
                indomainDef = indomainPreferredMin;
            }  
        }
    }
    
    public void addVariable(IntVar v, String h){
        if(h.equals("indomainMin")){
            map.put(v, indomainMin);
        }
        else if(h.equals("indomainMiddle")){
            map.put(v, indomainMiddle);
        }
        else if(h.equals("indomainMax")){
            map.put(v, indomainMax);
        }
        else if(h.equals("indomainRandom")){
            map.put(v, indomainRandom);
        }
        else if(h.equals("indomainPreferred")){
            map.put(v, indomainPreferredMin);
        }
    }
    
    public void addVariable(IntVar v, String p, String h){
        if(p.equals("indomainPreferred")){
            if(h.equals("indomainMiddle")){
                map.put(v, indomainPreferredMid);
            }
            else if(h.equals("indomainMax")){
                map.put(v, indomainPreferredMax);
            }
            else if(h.equals("indomainRandom")){
                map.put(v, indomainPreferredRnd);
            }
            else{
                map.put(v, indomainPreferredMin);
            }  
        }
    }
    
    @Override
    public int indomain(IntVar v) throws RuntimeException {
   
        
        Indomain<IntVar> sel = this.map.get(v);
        
        if(sel != null){
            //System.out.println(v.id+":"+sel.getClass().getName());
            return sel.indomain(v);
        }
        else{
            //System.out.println(v.id+":"+indomainDef.getClass().getName());
            return indomainDef.indomain(v);
        }
    }

}
