/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ascass.ui;

import com.csvreader.CsvWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import ascass.solving.CaspSolver;

/**
 *
 * @author erich
 */
public class Ascass {

    String prog;
    String instances;
    CsvWriter csvOutput;

    public Ascass(String prog, String instances) {
        this.prog = prog;
        this.instances = instances;
    }

    private ArrayList<String> getFilenames() {
        File dir = new File(this.instances);
        String[] children;
        FilenameFilter filter = new FilenameFilter() {

            public boolean accept(File dir, String name) {
                if (name.endsWith(".edb")) {
                    return true;
                } else if (name.endsWith(".dl")) {
                    return true; 
                } else if (name.endsWith(".ascass")) {
                    return true;
                } else if (name.endsWith(".asp")) {
                    return true;
                } else if (name.endsWith(".lp")) {
                    return true;
                } else if (name.endsWith(".pl")) {
                    return true;
                } else if (name.endsWith(".txt")) {
                    return true;
                } else {
                    return false;
                }
            }
        };
        children = dir.list(filter);

        return new ArrayList(java.util.Arrays.asList(children));
    }

    public void calculate() {
        System.out.println("\n running benchmark: " + this.instances);
        String outputFile = this.instances + "/" + "benchmark.csv";

        try {
            this.csvOutput = new CsvWriter(new FileWriter(outputFile), ';');
            this.csvOutput.setUseTextQualifier(false);
            this.write("file", "result", "asp", "cp");
        } catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
        }

        ArrayList <String> fn = this.getFilenames();
        for (int i = 0; i < fn.size(); i++) {
            System.out.println("\n file: " + fn.get(i));
            try {
                // ----------prog
                /*FileInputStream fstream = new FileInputStream(new File(this.prog));
                DataInputStream in = new DataInputStream(fstream);
                BufferedReader br = new BufferedReader(new InputStreamReader(in));
                String strLine;
                String fileContent = "";

                while ((strLine = br.readLine()) != null) {
                    fileContent += strLine + "\n";
                }

                in.close();
                
                //-----------instance
                fstream = new FileInputStream(new File(this.instances+"/"+fn.get(i)));
                in = new DataInputStream(fstream);
                br = new BufferedReader(new InputStreamReader(in));

                while ((strLine = br.readLine()) != null) {
                    fileContent += strLine + "\n";
                }

                in.close();
                
                CaspSolver solver = new CaspSolver(fileContent);*/
                
                String fileContent = readFile(this.prog,Charset.defaultCharset());
                fileContent += readFile(this.instances+"/"+fn.get(i),Charset.defaultCharset());
                
                CaspSolver solver = new CaspSolver(fileContent);
                int solved = solver.calcSolution();
                this.write(fn.get(i), solved+"", solver.aspTime+"", solver.cspTime+"");
            } catch (Exception e) {//Catch exception if any
                System.err.println(e.getMessage());
                e.printStackTrace();
            }
        }

        this.csvOutput.close();
    }

    private void write(String file, String result, String asp, String cp) {

        try {
            csvOutput.write(file);
            csvOutput.write(result);
            csvOutput.write(asp);
            csvOutput.write(cp);
            csvOutput.endRecord();

        } catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
        }
    }

    static String readFile(String path, Charset encoding) throws IOException {
        byte[] encoded = Files.readAllBytes(Paths.get(path));
        return new String(encoded, encoding);
    }
    
    public static void main(String [] args){
        if(args.length == 1 && args[0].equals("-gui") || args.length==0){
            try {
                for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                    if ("Nimbus".equals(info.getName())) {
                        javax.swing.UIManager.setLookAndFeel(info.getClassName());
                        break;


                    }
                }
            } catch (ClassNotFoundException ex) {
                java.util.logging.Logger.getLogger(MainFrame1.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
            } catch (InstantiationException ex) {
                java.util.logging.Logger.getLogger(MainFrame1.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
            } catch (IllegalAccessException ex) {
                java.util.logging.Logger.getLogger(MainFrame1.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
            } catch (javax.swing.UnsupportedLookAndFeelException ex) {
                java.util.logging.Logger.getLogger(MainFrame1.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
            }
            
            java.awt.EventQueue.invokeLater(new Runnable() {
                
                
                public void run() {
                    try{
                    System.out.println("working dir = "+(new File(".").getCanonicalPath()));
                    }catch(Exception e){
                        System.err.println(e.getMessage());
                        e.printStackTrace();
                    }
                    new MainFrame1().setVisible(true);
                }
            });
        }
        else if(args.length != 2){
            System.err.println("Wrong number of arguments. 'java -jar Ascass.jar %programPath/program% %pathToInstances%'");
        }
        else{
            Ascass b = new Ascass(args[0],args[1]);
            b.calculate();
        }
    }
}
