package checkers;




import java.io.File;
import java.io.IOException;

import ascass.solving.CaspSolver;
import scheduler.Changeover;
import scheduler.Job;
import scheduler.JobType;
import scheduler.Machine;
import scheduler.Scheduler;
import solvers.Solver;



public abstract class Checker{

	

	/** The scheduler to which the solver is attached. A solver can be attached to one Scheduler*/
	protected Scheduler scheduler;

											

	/** The Casp solver used in the ASCASS implementation of the jacob code. */
	protected CaspSolver caspSolver;



	public Checker(Scheduler scheduler){
		
		this.scheduler=scheduler;
		
	}




	
	/**
	 * Gets the scheduler binded with this Solver
	 *
	 * @return the scheduler
	 */
	public Scheduler getScheduler() {
		return scheduler;
	}





	/**
	 * This method must be implemented for every class that extends the {@link Solver Solver class}
	 *
	 * @return the int
	 */
	public abstract int check();


	
	





	/**
	 * Gets the casp solver used by the ASCASS implementation. 
	 *
	 * @return the casp solver
	 */
	public CaspSolver getCaspSolver() {
		return caspSolver;
	}

}
