package coders;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.LineIterator;

import scheduler.Changeover;
import scheduler.Group;
import scheduler.Job;
import scheduler.JobType;
import scheduler.Machine;
import scheduler.Scheduler;

public class Decoder_IncrementalScheduling extends Decoder{
	
	
	private Collection<Job> jobCollection;
	private Collection<Machine> machineCollection;

	public Decoder_IncrementalScheduling(Scheduler scheduler) {
		super(scheduler);
		// TODO Auto-generated constructor stub
	}

	@Override
	public String decode() {
		// TODO Auto-generated method stub
		jobCollection= scheduler.getJobs().values();
		machineCollection = scheduler.getMachines().values();
		
		StringBuilder sb=new StringBuilder();
		sb.append("\n");
		sb.append(maxTimeToString());
		sb.append(currentTimeToString());
		sb.append(machinesToString());
		sb.append(jobsToString());
		sb.append(precendecesToString());
		sb.append(jobtodevicesToString());
		sb.append(joblengthsToString());
		sb.append(deadlinesToString());
		
		return sb.toString();
	}
	
	
	@Override
	public String decode(Group g) {
		// TODO Auto-generated method stub
		jobCollection= g.getJobsInGroup();
		machineCollection = g.getMachinesInGroup();
		
		StringBuilder sb=new StringBuilder();
		sb.append("\n");
		sb.append(maxTimeToString());
		sb.append(currentTimeToString());
		sb.append(machinesToString());
		sb.append(jobsToString());
		sb.append(precendecesToString());
		sb.append(jobtodevicesToString());
		sb.append(joblengthsToString());
		sb.append(deadlinesToString());
		
		return sb.toString();
	}
	
	
	

	@Override
	public String decodeSolution() {
		// TODO Auto-generated method stub
		StringBuilder sb= new StringBuilder();
		for (Job j : scheduler.getJobs().values())
			if (j.getStartsAt()!=null){
				sb.append("cspconstr(start("+j.getId()+","+j.getType()+"),eq,"+j.getStartsAt()+").\n");
				if (j.getAssignedToMachine()==null)
					throw new RuntimeException("Job "+j.getId()+" has starting time "+j.getStartsAt()+" but it is not assigned to a machine");
				else
					sb.append("cspconstr(assign("+j.getId()+","+j.getType()+"),eq,"+j.getAssignedToMachine().getInstance()+").\n");
			}
		return sb.toString();
				
		
	}
	
	
	@Override
	public String decodeSolutionAtoms() {
		// TODO Auto-generated method stub
		StringBuilder sb= new StringBuilder();
		for (Job j : scheduler.getJobs().values())
			if (j.getStartsAt()!=null){
				sb.append("start("+j.getId()+","+j.getType()+","+j.getStartsAt()+").\n");
				sb.append("finished("+j.getId()+","+j.getType()+","+j.getFinishesAt()+").\n");
				if (j.getAssignedToMachine()==null)
					throw new RuntimeException("Job "+j.getId()+" has starting time "+j.getStartsAt()+" but it is not assigned to a machine");
				else
					sb.append("assign("+j.getId()+","+j.getType()+","+j.getAssignedToMachine().getInstance()+").\n");
			}
		return sb.toString();
				
		
	}
	
	
	


	private String maxTimeToString(){
		return Constants_IS.MAXTIME+"("+scheduler.getMaxTime()+").\n";
	}
	
	private String currentTimeToString(){
		return Constants_IS.CURRENTTIME+"("+scheduler.getCurrentTime()+").\n";
	}
	
	private String jobsToString(){

		int jobs=0;
		StringBuilder sb=new StringBuilder();
		
		Iterator<Job> jobIterator=jobCollection.iterator();
		
		while(jobIterator.hasNext()){
			Job j=jobIterator.next();
			sb.append(Constants_IS.JOBS+"("+j.getId()+").\n");
			jobs++;
		}
	
		return sb.toString();
	}
	
	
	
	
	
	
	private String machinesToString(){
		StringBuilder sb= new StringBuilder();
		Iterator<Machine> machineIterator=machineCollection.iterator();
		List<JobType> alreadyAdded=new ArrayList<JobType>();
		
		while(machineIterator.hasNext()){
			
			Machine m=machineIterator.next();
			JobType currentType=m.getCurrentType();
			if (!alreadyAdded.contains(currentType)){
				Iterator<Machine> machineInnerIterator=machineCollection.iterator();
				sb.append(Constants_IS.MACHINEDEVICE+"("+currentType.toString()+").\n");
				int instances=0;
				while(machineInnerIterator.hasNext()){
					Machine inner=machineInnerIterator.next();
					if (inner.getCurrentType().equals(currentType)){
						instances++;
						sb.append("cspvar("+Constants_IS.DEVICEINSTANCETOMACHINEID+"("+currentType.toString()+","+instances+"),"+inner.getId()+","+inner.getId()+").\n");
					}
				}
				sb.append(Constants_IS.MACHINEINSTANCES+"("+currentType.toString()+","+instances+").\n");
				alreadyAdded.add(currentType);
			}
		}
		return sb.toString();
	}
	
	
	
	
	
	private String deadlinesToString() {
		StringBuffer sb=new StringBuffer();
		Iterator<Job> it=jobCollection.iterator();
		while(it.hasNext()){
			Job j=it.next();
			if(j.hasDeadline())
				sb.append(Constants_IS.DEADLINE+"("+j.getId()+","+j.getDeadline()+").\n");
		}
		return sb.toString();
	}

	
	private String importancesToString() {
		StringBuffer sb=new StringBuffer();
		Iterator<Job> it=jobCollection.iterator();
		while(it.hasNext()){
			Job j=it.next();
			
				sb.append(Constants_IS.IMPORTANCE+"("+j.getId()+","+j.getImportance()+").\n");
		}
		return sb.toString();
	}

	private String precendecesToString() {
		StringBuffer sb=new StringBuffer();
		Iterator<Job> it=jobCollection.iterator();
		while(it.hasNext()){
			Job j=it.next();
			if(j.hasPreviousJob())
				sb.append(Constants_IS.PRECEDENCE+"("+j.getPreviousJob().getId()+","+j.getId()+").\n");
		}
		return sb.toString();
	}
	
	


	
	

	
	private String joblengthsToString() {
		StringBuffer sb=new StringBuffer();
		Iterator<Job> it=jobCollection.iterator();
		while(it.hasNext()){
			Job j=it.next();
			sb.append(Constants_IS.JOBLENGTH+"("+j.getId()+","+j.getMaxJobLength()+").\n");
				
		}
		return sb.toString();
	}
	
	
	private String jobtodevicesToString() {
		StringBuffer sb=new StringBuffer();
		Iterator<Job> it=jobCollection.iterator();
		while(it.hasNext()){
			Job j=it.next();
			sb.append(Constants_IS.JOBTYPE+"("+j.getId()+","+j.getType()+").\n");
				
		}
		return sb.toString();
	}
	
	

	
	
	
	
	
	

}
