package coders;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.LineIterator;
import org.jacop.core.IntVar;
import org.jacop.core.Var;

import scheduler.Group;
import scheduler.Job;
import scheduler.JobType;
import scheduler.Machine;
import scheduler.Scheduler;
public class Encoder_IncrementalScheduling extends Encoder{

	
	private File input;
	private LineIterator it;
	
	

	
	
	public Encoder_IncrementalScheduling(Scheduler scheduler) {
		super(scheduler);
		input= new File(scheduler.getInputPath());
		// TODO Auto-generated constructor stub
	}

	@Override
	public void encode() {

		
		parseCurrTime();
		//System.out.println("Current Time Parsed");
		parseJobs();
		//System.out.println("Jobs Parsed");
		parseJobType();
		//System.out.println("Job Types Parsed");
		parseMachines();
		//System.out.println("Machines Parsed");
		parsePrecedence();
		//System.out.println("Precedences Parsed");
		parseDeadlines();
		//System.out.println("Deadlines Parsed");
		parseImportances();
		//System.out.println("Importances Parsed");
		parseJobLength();
		//System.out.println("Job Lengths Parsed");
		parseMaxTime();
		//System.out.println("Max Time Parsed");
	}

	
	/**
	 * Gathers the information about the starting time and the assignment to machine of the jobs 
	 * Used after the solver is launched, to parse the solution to the Scheduler
	 * The solution format of the cspvar is the following
	 * start(J,D) =T			J=JobID, D=Device, T=Starting time of the Job
	 * assign(J,D)=I			J=JobID, D=Device, I=Instance of the Device D
	 */
	@Override
	public void encodeSolution(Var[] vars) {
		// TODO Auto-generated method stub
		String notParsedArgs;
		Job jobFromVar;
		Machine machineFromVar;
		int startTime,instance;
		HashMap<String, Machine> machineInSolution=new HashMap<String, Machine>();
		//this step is necessary to create a correspondance between the device istance in the solution and the machine id in the scheduler
		for (Var v : vars) {
			
			if (v != null
					&& v.id.startsWith(Constants_IS.DEVICEINSTANCETOMACHINEID + "(")) {
				notParsedArgs = v.id.substring(v.id.indexOf('(') + 1,
						v.id.indexOf(')'));
				machineInSolution.put(notParsedArgs, scheduler.getMachines().get(((IntVar) v).value()));
				
				
			}
			
			
			
			
		}
		
		for (Var v : vars) {

			if (v != null
					&& v.id.startsWith(Constants_IS.START_CSPVAR + "(")) {
				startTime = ((IntVar) v).value();
				notParsedArgs = v.id.substring(v.id.indexOf('(') + 1,
						v.id.indexOf(','));
				jobFromVar = scheduler.getJobs().get(notParsedArgs);
				jobFromVar.setStartsAt(startTime);
				jobFromVar.setFinishesAt(startTime+jobFromVar.getMaxJobLength());
			}

			if (v != null
					&& v.id.startsWith(Constants_IS.ASSIGN_CSPVAR + "(")) {
				instance = ((IntVar) v).value();
				notParsedArgs = v.id.substring(v.id.indexOf('(') + 1,
						v.id.indexOf(','));
				jobFromVar = scheduler.getJobs().get(notParsedArgs);
				notParsedArgs = v.id.substring(v.id.indexOf(',') + 1,
						v.id.indexOf(')'));
				machineFromVar = machineInSolution.get(notParsedArgs+","+ instance);
				jobFromVar.setAssignedToMachine(machineFromVar);
				machineFromVar.addJob(jobFromVar);
		


			}
		
		}
	}
	
	
	
	@Override
	public void encodeGroups(Var[] vars) {
		// TODO Auto-generated method stub
		
		String notParsedArgs;
		String[] args;
		Job jobFromVar;
		Machine machineFromVar;
		int groupId,numInstances;
		
	
			  for(Var v: vars){
		        
				  if(v!=null && v.id.startsWith(Constants_IS.MACHINEGROUP_CSPVAR+ "(")){
					
					  groupId=((IntVar)v).value();
					
					  Group g=scheduler.getGroups().get(groupId);
					  if (g==null){
						  g=new Group(groupId);
						  scheduler.addGroup(g);
					  }
					  notParsedArgs = v.id.substring(v.id.indexOf('(') + 1,
							v.id.indexOf(')'));
					  args=notParsedArgs.split(",");
					  numInstances= Integer.parseInt(args[1]);
					 // for (int i=1;i<=numInstances;i++){
						  machineFromVar = scheduler.getMachineFromDeviceInstance(args[0], numInstances);
					  	  g.getMachinesInGroup().add(machineFromVar);
					 // }
					   
				  }
				  
			  }
			  
			  
		  for(Var v: vars){
		        
			  if(v!=null && v.id.startsWith(Constants_IS.JOBGROUP_CSPVAR+ "(")){
				  
				  groupId=((IntVar)v).value();
				  notParsedArgs = v.id.substring(v.id.indexOf('(') + 1,
							v.id.indexOf(')'));
				  jobFromVar = scheduler.getJobs().get(notParsedArgs);
				
				  Group g=scheduler.getGroups().get(groupId);
				  if (g==null){
					  g=new Group(groupId);
					  scheduler.addGroup(g);
				  }
				  g.getJobsInGroup().add(jobFromVar);
				  
				  
				//  System.out.println("group size "+g.getJobsInGroup().size()+" on "+jobs);
			  }
				  
				  
			
				  
				  
			
			  }
			  
			  
			}
		
	







	
	

/**
 * MaxTime should always be parsed after the jobs and the joblengths, because 
 * if the maxTime is not specified in the file, a theoretical value is taken, the sum of all the joblengths
 * 
 */
	
	private void parseMaxTime() {
		int maxtime=-1;
		
		try {
			it = FileUtils.lineIterator(input, "UTF-8");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		String line = "";
		do {
			line = it.nextLine();
			if (line.startsWith(Constants_IS.MAXTIME + "(")) {
				String notParsedArgs = line.substring(line.indexOf('(') + 1,
						line.indexOf(')'));
				maxtime=Integer.parseInt(notParsedArgs);
				scheduler.setMaxTime(maxtime);
				break;
			}
		}while (it.hasNext());
		
		if (maxtime==-1)
			scheduler.generateTheoreticalMaxTime();
	}
		
		
		
		private void parseCurrTime() {

		
			try {
				it = FileUtils.lineIterator(input, "UTF-8");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			String line = "";
			do {
				line = it.nextLine();
				if (line.startsWith(Constants_IS.CURRENTTIME + "(")) {
					String notParsedArgs = line.substring(line.indexOf('(') + 1,
							line.indexOf(')'));
					scheduler.setCurrentTime(Integer.parseInt(notParsedArgs));
					break;
				}
			}while (it.hasNext());
		}

		private void parseJobs() {

			
			try {
				it = FileUtils.lineIterator(input, "UTF-8");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			String currentLine = "";
			do	{
				currentLine = it.nextLine();
				if (currentLine.startsWith(Constants_IS.JOBS + "(")) {

					String notParsedArgs = currentLine.substring(
							currentLine.indexOf('(') + 1, currentLine.indexOf(')'));
					String[] args = notParsedArgs.split(",");
				
					scheduler.getJobs().put(args[0], new Job(args[0]));

				}
			}while (it.hasNext());
		}

		private void parseMachines() {

			
			try {
				it = FileUtils.lineIterator(input, "UTF-8");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			String currentLine = "";
			do {
				currentLine = it.nextLine();
				if (currentLine.startsWith(Constants_IS.MACHINEINSTANCES + "(")) {

					String notParsedArgs = currentLine.substring(
							currentLine.indexOf('(') + 1, currentLine.indexOf(')'));
					String[] args = notParsedArgs.split(",");
					int nMachines = scheduler.getMachines().size();
					for (int i = 0; i < Integer.parseInt(args[1]); i++) {
						int id = nMachines + i;
						Machine m = new Machine(id,args[0],i+1);
						m.setCurrentType(new JobType(args[0]));
						scheduler.getMachines().put(id, m);
					}

				}
			}while (it.hasNext());
		}

		
		
		private void parsePrecedence() {

			
			try {
				it = FileUtils.lineIterator(input, "UTF-8");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			String currentLine = "";
			do {
				currentLine = it.nextLine();
				if (currentLine.startsWith(Constants_IS.PRECEDENCE+"(")) {

					String notParsedArgs = currentLine.substring(
							currentLine.indexOf('(') + 1, currentLine.indexOf(')'));
					String[] args = notParsedArgs.split(",");
					Job currentJob = scheduler.getJobs().get(args[1]);
					Job previousJob = scheduler.getJobs().get(args[0]);
					currentJob.setPreviousJob(previousJob);

				
				}
			}while (it.hasNext()); 
		}
		
		
		
		private void parseDeadlines() {

			try {
				it = FileUtils.lineIterator(input, "UTF-8");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			String currentLine = "";
			while (it.hasNext()) {
				currentLine = it.nextLine();
				if (currentLine.startsWith(Constants_IS.DEADLINE+"(")) {

					String notParsedArgs = currentLine.substring(
							currentLine.indexOf('(') + 1, currentLine.indexOf(')'));
					String[] args = notParsedArgs.split(",");
					Job currentJob = scheduler.getJobs().get(args[0]);
					currentJob.setDeadline(Integer.parseInt(args[1]));
		
				}
			}
			
		
		}

		private void parseImportances() {

			try {
				it = FileUtils.lineIterator(input, "UTF-8");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			String currentLine = "";
			while (it.hasNext()) {
				currentLine = it.nextLine();
				if (currentLine.startsWith(Constants_IS.IMPORTANCE+"(")) {

					String notParsedArgs = currentLine.substring(
							currentLine.indexOf('(') + 1, currentLine.indexOf(')'));
					String[] args = notParsedArgs.split(",");
					Job currentJob = scheduler.getJobs().get(args[0]);
					currentJob.setImportance(Integer.parseInt(args[1]));
		
				}
			}
			
		}


		
		private void parseJobType() {

		
			try {
				it = FileUtils.lineIterator(input, "UTF-8");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			String currentLine = "";
			do {
				currentLine = it.nextLine();
				if (currentLine.startsWith(Constants_IS.JOBTYPE+"(")) {

					String notParsedArgs = currentLine.substring(
							currentLine.indexOf('(') + 1, currentLine.indexOf(')'));
					String[] args = notParsedArgs.split(",");
					Job currentJob = scheduler.getJobs().get(args[0]);
					currentJob.setType(new JobType(args[1]));
		
				}
			} while (it.hasNext());
		}
		
		

		private void parseJobLength() {

			try {
				it = FileUtils.lineIterator(input, "UTF-8");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			String currentLine = "";
			do {
				currentLine = it.nextLine();
				if (currentLine.startsWith(Constants_IS.JOBLENGTH+"(")) {

					String notParsedArgs = currentLine.substring(
							currentLine.indexOf('(') + 1, currentLine.indexOf(')'));
					String[] args = notParsedArgs.split(",");
					Job j = scheduler.getJobs().get(args[0]);
					for (Machine m: scheduler.getMachines().values())
						if (m.getCurrentType().equals(j.getType()))
							j.addPossibleMachine(m, Integer.parseInt(args[1]));
					
		
				}
			} while (it.hasNext());
			
		}

		
		

	
		
	
		
		
		private void calculateLots(){
			int lotId=1;
			for (Job j : scheduler.getJobs().values())
				if (j.hasDeadline()){
					j.setLot(lotId);
					while(j.hasPreviousJob()){
						j=j.getPreviousJob();
						j.setLot(lotId);
					}
					lotId++;
				}
			
		}
		
		
		
		
		public Machine fromInstDevToMachine(String device, int instance){
			ArrayList<Integer> machines= new ArrayList<Integer>(); 
			for (Machine m: scheduler.getMachines().values())
				if (m.getCurrentType().equals(device))
					machines.add(m.getId());
			Collections.sort(machines);
			int id=machines.get(instance-1);
			return  scheduler.getMachines().get(id);
		}


	
	
}
