package launchers;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import checkers.Checker_IncrementalScheduling;
import scheduler.Scheduler;
import solvers.CumulativeIndipendent_IncrementalScheduling;
import solvers.Solver;
import solvers.Cumulative_IncrementalScheduling;
import splitters.Lotwise_IncrementalScheduling;
import splitters.Machinewise_IncrementalScheduling;
import splitters.Typewise_IncrementalScheduling;

public class SolverEvaluation {

	
	
	private static final String INPUTFOLDER="/input/";
	
	private static final String OUTPUTFOLDER="/output/";

	
	static String outputString = "";
	static String filename = "";
	static String currentFolder = null;

	/**
	 * the first argument, if present, is the splitting factor
	 * the second argument, if present, is the timeout for the solver 
	 * @param args
	 */
	public static void main(String[] args) {

		String splittingFactor="";

		outputString = "";
		filename = "";
		currentFolder = null;
		try {

			currentFolder = new File(".").getCanonicalPath();
			System.out.println(currentFolder);
			outputString = currentFolder + OUTPUTFOLDER;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

		}
		// for every input

			File f = new File(currentFolder + INPUTFOLDER
					+ "/solverEvaluation");
			
			for (File j : f.listFiles())
				if (j.isDirectory()) 
					for (File k : j.listFiles())
						if (k.isDirectory()) 
							for (File i : k.listFiles()) {
						
								String folderName=j.getName();
								System.out.println(j.getAbsolutePath());
								
								String numMachines=k.getName().substring(0, 3);
								String numJobs=folderName.substring(0, 5);
								if (args.length>0)
									splittingFactor=args[0];
								else
									splittingFactor=folderName.substring(folderName.length()-3);
								
						
								String length= (i.getName().startsWith("short"))? "short":"long";
								String type=extractType(i.getName());
								System.out.println("file= "+i.getName()+"  type= "+type+"  numMachines= "+numMachines+"  numJobs= "+numJobs+"  splittingFactor= "+splittingFactor);
								
								Scheduler s = new Scheduler(i.getAbsolutePath());
								s.parseInput();
								s.setSplitter(new Machinewise_IncrementalScheduling(s));
								s.getSplitter().split();
								s.PrintGroups();
								long timer=System.currentTimeMillis();
								if (args.length>1)
									s.setSolver(new CumulativeIndipendent_IncrementalScheduling(s,Integer.parseInt(args[1])));
								else
									s.setSolver(new CumulativeIndipendent_IncrementalScheduling(s));
								
								try{
								int solution=s.getSolver().solve();
								timer=System.currentTimeMillis()-timer;
								s.setChecker(new Checker_IncrementalScheduling(s));
								int check=s.getChecker().check();
								s.printSolution();
								generateCsvFile(outputString+"SolverEvaluationMTWRSplitPerJob.csv",new String[]{i.getName(),length,type,numMachines,numJobs,splittingFactor,timer+"",s.getMakeSpan()+"",solution+"",check+""});
								
								}
								catch (java.lang.OutOfMemoryError memory){
									timer=System.currentTimeMillis()-timer;
									generateCsvFile(outputString+"SolverEvaluationMTWRSplitPerJob.csv",new String[]{i.getName(),length,type,numMachines,numJobs,splittingFactor,timer+"",0+"",-3+"",0+""});
									memory.printStackTrace();
								}
								}
								
							
						

				

	}
	
	private static String extractType(String s){
		
		int index1= s.indexOf('-')+1;
		int index2= index1 + s.substring(index1).indexOf('-') ;
		return s.substring(index1,index2);
	}

	private static void printResults( Scheduler s, String title,int d) {
			
			
			System.out.println("___________________________________");
			System.out.println("___________"+title+ " factor= "+d+" is started___________");
			
			
			s.parseInput();
			s.setSplitter(new Lotwise_IncrementalScheduling(s,5));
			s.getSplitter().split();
			s.setSolver(new Cumulative_IncrementalScheduling(s));
			s.getSolver().solve();
			

		}
	
	
	
	
	
	private static void generateCsvFile(String sFileName, String[] args) {
		
		char separator = ';';
		try {
			FileWriter writer = new FileWriter(sFileName, true);
			
			for (String s: args){
				writer.append(s);
				writer.append(separator);
			}
			writer.append('\n');


			writer.flush();
			writer.close();
		
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
}
