package launchers;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import scheduler.Group;
import scheduler.Machine;
import scheduler.Scheduler;
import solvers.Solver;
import solvers.Cumulative_IncrementalScheduling;
import splitters.Lotwise_IncrementalScheduling;
import splitters.Machinewise_IncrementalScheduling;
import splitters.Splitter;
import splitters.Typewise_IncrementalScheduling;

public class SplitterEvaluation {

	
	
	private static final String INPUTFOLDER="/input/";
	
	private static final String OUTPUTFOLDER="/output/";

	
	static String outputString = "";
	static String filename = "";
	static String currentFolder = null;

	
	public static void main(String[] args) {



		outputString = "";
		filename = "";
		currentFolder = null;
		try {

			currentFolder = new File(".").getCanonicalPath();
			System.out.println(currentFolder);
			outputString = currentFolder + OUTPUTFOLDER;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

		}
		// for every input

			File f = new File(currentFolder + INPUTFOLDER
					+ "/splitterEvaluation");
			
			for (File j : f.listFiles())
				if (j.isDirectory()) 
					for (File k : j.listFiles())
						if (k.isDirectory()) 
							for (File i : k.listFiles()) {
								String folderName=j.getName();
								System.out.println(j.getAbsolutePath());
								String numMachines=k.getName().substring(0, 3);
								String numJobs=folderName.substring(0, 5);
								int splittingFactor=Integer.parseInt(folderName.substring(folderName.length()-3));
								String length= (i.getName().startsWith("short"))? "short":"long";
								String type=extractType(i.getName());
								
								System.out.println("file= "+i.getName()+"  type= "+length+"  numMachines= "+numMachines+"  type= "+type+"  numJobs= "+numJobs+"  splittingFactor= "+splittingFactor);
								
								Scheduler s = new Scheduler(i.getAbsolutePath());
								s.parseInput();
								
								List<Splitter> splitters= new ArrayList<Splitter>(
									    Arrays.asList(new Machinewise_IncrementalScheduling(s)));
								int ind=0;
								for (Splitter sp: splitters){
									ind++;
									s.setSplitter(sp);
									long timer=System.currentTimeMillis();
									s.getSplitter().split();
									//s.PrintGroups();
									for (Group g: s.getGroups().values()){
										
										System.out.println("_____________________GROUP"+g.getId());
											for (Machine m: g.getMachinesInGroup())
												System.out.println(m.toString());
									}
								
									String title = "";
									switch(ind){
										case 1:
											title="Machinewise";
											break;
										case 2:
											title="Typewise";
											break;
										case 3:
											title="Lotwise";
											break;
										
									}
									
									
									
									timer=System.currentTimeMillis()-timer;
									
									generateCsvFile(outputString+"SplitterEvaluationMachinewise2.csv",new String[]{title,i.getName(),length,type,numMachines,numJobs,sp.getSplittingFactor()+"",timer+""});
								}
							}
						

				

	}
	
	
	private static String extractType(String s){
		
		int index1= s.indexOf('-')+1;
		int index2= index1 + s.substring(index1).indexOf('-') ;
		return s.substring(index1,index2);
	}


	private static void printResults( Scheduler s, String title,int d) {
			
			
			System.out.println("___________________________________");
			System.out.println("___________"+title+ " factor= "+d+" is started___________");
			
			
			s.parseInput();
			s.setSplitter(new Lotwise_IncrementalScheduling(s,5));
			s.getSplitter().split();
			
			s.printAllContent();

		}
	
	
	
	
	
	private static void generateCsvFile(String sFileName, String[] args) {
		
		char separator = ';';
		try {
			FileWriter writer = new FileWriter(sFileName, true);
			
			for (String s: args){
				writer.append(s);
				writer.append(separator);
			}
			writer.append('\n');


			writer.flush();
			writer.close();
		
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
}
