/**
 */
package scheduler;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;



import scheduler.Group;
import scheduler.JobType;
import scheduler.Machine;

/**
 * <!-- begin-user-doc -->
 *The class describe all the components of a Job  
 * <!-- end-user-doc -->

 */
public class Job {


	protected String id;



	protected JobType type;
	
	

	
	protected Integer deadline;




	protected Integer startsAt;

	/**
	 this map use the machine as a key and gives back the length of the job on that machine as a value
	 */
	protected HashMap<Machine, Integer> jobLength;

	

	/**
	couplingLength specifies the length of the coupling constraint, this is relevant only in the Infineon scheduling problem
	 */
	protected Integer couplingLength;

	protected boolean alreadyFinished = false;
	

	protected Integer lot;

	protected Group group;
	
	protected Job coupledJob;
	
	protected Job previousJob;

	protected Machine assignedTo;

	protected Integer finishesAt;

	protected static final int IMPORTANCE_DEFAULT = 0;
	
	protected int importance=IMPORTANCE_DEFAULT;

	protected boolean currentJob=false;

	/**
	 workflow specifies the position of the job in the workflow of a lot (the first job of a lot, the second, etc...)
	 */
	protected int workflow = -1;
	
	
	
	
	public Job(String id) {
		super();
		setId(id);
		this.jobLength= new HashMap<Machine, Integer>();
	}

	public Job(String id, JobType type,int workflow) {
		super();
		setId(id);
		setType(type);
		setWorkflow(workflow);
		this.jobLength= new HashMap<Machine, Integer>();
	}


	
	public String getId() {
		return id;
	}


	public void setId(String newId) {
		
		id = newId;
	}


	public Job getPreviousJob() {
		return previousJob;
	}



	public void setPreviousJob(Job newPreviousJob) {
		previousJob = newPreviousJob;
	}


	public JobType getType() {
		return type;
	}

	public void setType(JobType newType) {
		type = newType;
	}

	
	public int getDeadline() {
		return deadline;
	}


	public void setDeadline(int newDeadline) {
		deadline = newDeadline;
	}

	
	public boolean hasDeadline(){
		return this.deadline != null;
	}
	
	
	
	public int getImportance() {
		return importance;
	}

	public void setImportance(int importance) {
		this.importance = importance;
	}


	public boolean hasPreviousJob(){
		return this.previousJob!= null;
	}
	
    /**
     * gets the starting time of the job
     * @return the starting time of the job, if not specified, returns null
     */
	public Integer getStartsAt() {
		return startsAt;
	}

	
	public void setStartsAt(int newStartsAt) {
		startsAt = newStartsAt;
	}


	public int getMaxJobLength() {
		int max=0;
		for (Integer i:jobLength.values())
			if (i>max)
				max=i;
		return max;
	}
	
	public HashMap<Machine,Integer> getJobLength() {
		
		return jobLength;
	}

	
	public Machine getAssignedToMachine() {
		return assignedTo;
	}


	

	public void setAssignedToMachine(Machine newAssigned2Machine) {
		assignedTo = newAssigned2Machine;
	}

	
	public int getFinishesAt() {
		return finishesAt;
	}

	
	public void setFinishesAt(int newFinishAt) {
		finishesAt = newFinishAt;
	}

	
	public Job getCoupledJob() {
		return coupledJob;
	}



	public void setCoupledJob(Job newCoupledJob) {
		coupledJob = newCoupledJob;
		}

	
	
	public void setCoupledJob(Job job2, int length) {
		setCoupledJob(job2);
		setCouplingLength(length);
	}
	

	public int getCouplingLength() {
		return couplingLength;
	}

	


	public void setCouplingLength(int newCouplingLength) {
		couplingLength = newCouplingLength;
		}

	
	public boolean isAlreadyFinished() {
		return alreadyFinished;
	}

	
	public void setAlreadyFinished(boolean newAlreadyFinished) {
		alreadyFinished = newAlreadyFinished;}

	
	public Group getGroup() {
		return group;
	}



	
	public void setGroupId(Group newGroupId) {
		group = newGroupId;
	}



	public int getLot() {
		return lot;
	}

	
	public void setLot(int lot) {
		this.lot = lot;
	}

	
	

	public int getWorkflow() {
		return workflow;
	}

	public void setWorkflow(int workflow) {
		this.workflow = workflow;
	}
	
	
	

	@Override
	public String toString() {


		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (id: ");
		result.append(id);
		result.append(", type: ");
		result.append(type.toString());
		result.append(", lot: ");
		result.append(lot);
		result.append(", deadline: ");
		result.append(deadline);
		result.append(", startsAt: ");
		result.append(startsAt);
		result.append(", coupling: ");
		result.append(coupledJob==null ? "NO" : coupledJob.getId());
		result.append(", alreadyFinished: ");
		result.append(alreadyFinished);
		result.append(", finishAt: ");
		result.append(finishesAt);
		result.append(", PossibleMachines: ");
		result.append(getJobLength().size());
		result.append(')');
		return result.toString();
	}

	/**
	 * Adds one machine to the list of machines that can handle this job, and the length of the job on that machine
	 */
	public void addPossibleMachine(Machine machine, int value) {
		// TODO Auto-generated method stub
		jobLength.put(machine, value);
	}


	public boolean hasCoupling() {
		return this.couplingLength!=null;
	}

	
	public void assignToMachine(Machine m, int startTime) {
		// TODO Auto-generated method stub
		setAssignedToMachine(m);
		setStartsAt(startTime);
	}

	
	public boolean isCurrentJob() {
		return currentJob;
	}

	public void setCurrentJob(boolean currentJob) {
		this.currentJob = currentJob;
	}




	@Override
	public boolean equals(Object other) {
		 if (other == null) return false;
		 if (other == this) return true;
		 if (!(other instanceof Job))return false;
		    
		 Job otherMyClass = (Job)other;
		    
		 return this.getId().equals(otherMyClass.getId());
	}

	



} 
