package scheduler;



/**
 * <!-- begin-user-doc -->
 * The '<em><b>JobType</b></em>' class specifies the type of every job. 
 * In particular '<em><b>task</b></em>' identifies the operation that can be performed on that job, for example inking, cutting etc.
 * '<em><b>product</b></em>' identifies the type of the product. For example in the Infineon case the inking task can be performed on product 12, 8 and 6 inch wide
 * In the case of incremental scheduling there is just one product type per task, therefore '<em><b>product</b></em>' will assume the default value.  
 * <!-- end-user-doc -->
 */
public class JobType implements Comparable<JobType>{
	private static final String PRODUCT_DEFAULT=""; 
	
	private String product=PRODUCT_DEFAULT;
	

	private String task;
	
	public JobType(String product,String task){
		this.product=product;
		this.task=task;
	}
	
	public JobType(String task){
		this.product=PRODUCT_DEFAULT;
		this.task=task;
	}

	
	public String getProduct() {
		return product;
	}


	public String getTask() {
		return task;
	}

	

	public String toString(){
		if (product.equals(PRODUCT_DEFAULT))
			return task;
		else
			return product+","+task;
	}


	@Override
	public boolean equals(Object arg0) {
		if (this==arg0)
			return true;
		
		if (arg0 instanceof JobType)
			return this.toString().equals(((JobType)arg0).toString());
		else
			return false;
	}


	@Override
	public int compareTo(JobType arg0) {
		return this.toString().compareTo(arg0.toString());
	}

	

}

