/**
 */
package scheduler;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;






import scheduler.Changeover;
import scheduler.Group;
import scheduler.Job;
import scheduler.JobType;

/**
 *Machine includes all the features of a machine in the scheduling problem of infineon and incremental scheduling
 */
public class Machine {

	protected static final int ID_DEFAULT = -1;

	
	protected int id = ID_DEFAULT;

	protected int instance;
	
	protected String device;
	/**
	 * This list states the time interval in which the machine is not available
	 */
	protected List<Interval> machineDownIntervals;

	protected JobType currentType;
	
	protected List<Job> scheduledJobs;
	
	protected List<JobType> scheduledJobTypes;

	protected List<String> scheduledJobTasks;
	/**
	 * This variable specifies the group the machine belongs to
	 */
	protected Group group;
	
	

	/**
	 * The method instantiate the machine with the specified id. It also initialize the Lists of machineDownIntervals, scheduledJobs, 
	 */
	public Machine(int id) {
		super();
		this.id=id;
		machineDownIntervals= new ArrayList<Interval>();
		scheduledJobs= new ArrayList<Job>();
		scheduledJobTypes= new ArrayList<JobType>();
		scheduledJobTasks= new ArrayList<String>();
	}
	
	
	public Machine(int id, String device, int instance) {
		super();
		this.id=id;
		this.device=device;
		this.instance=instance;
		machineDownIntervals= new ArrayList<Interval>();
		scheduledJobs= new ArrayList<Job>();
		scheduledJobTypes= new ArrayList<JobType>();
		scheduledJobTasks= new ArrayList<String>();
	}


	public int getId() {
		return id;
	}


	
	public int getInstance() {
		return instance;
	}


	public String getDevice() {
		return device;
	}


	public List<Interval> getMachineDownIntervals() {
		return machineDownIntervals;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setMachineDownIntervals(List<Interval> newAvailability) {
		machineDownIntervals = newAvailability;}

	
	public JobType getCurrentType() {
		return currentType;
	}

	
	public void setCurrentType(JobType jobType) {
		currentType = jobType;
		}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public List<Job> getScheduledJobs() {
		return scheduledJobs;
	}

	

	
	public List<JobType> getScheduledTypes() {
		// TODO Auto-generated method stub
		return scheduledJobTypes;
	}

	

	public List<String> getScheduledJobTasks() {
		return scheduledJobTasks;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Group getGroup() {
	
		return group;
	}



	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setGroup(Group newGroupId) {
		group = newGroupId;}

	
	public void addMachineDownInterval(int value1, int value2) {
		
		Interval i=new Interval(value1,value2);
		addMachineDownInterval(i);
	}
	
	public void addMachineDownInterval(Interval i) {
		
		machineDownIntervals.add(i);
		machineDownIntervals.sort(new IntervalComparator());
	}
	

	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void addJob(Job j) {
		if(!scheduledJobs.contains(j))
			scheduledJobs.add(j);
		scheduledJobs.sort(new JobComparator());
		//Job lastJob=scheduledJobs.get(scheduledJobs.size()-1);
		setCurrentType(getLastScheduledJob().getType());
		addJobType(j.getType());
		addJobTask(j.getType().getTask());
	}


	public void sortScheduledJobs(){
		scheduledJobs.sort(new JobComparator());
		if(getLastScheduledJob()!=null)
			setCurrentType(getLastScheduledJob().getType());
	}
	
	public void addJobType(JobType j) {
		if (!scheduledJobTypes.contains(j)){
			scheduledJobTypes.add(j);
			scheduledJobTypes.sort(new JobTypeComparator());
		}
	}
	
	public void addJobTask(String j) {
		if (!scheduledJobTasks.contains(j)){
			scheduledJobTasks.add(j);
			scheduledJobTasks.sort(String.CASE_INSENSITIVE_ORDER);
			 
		}
	}

	
	/**
	 it returns the job scheduled in the last position
	 if there is no job scheduled in the machine, it returns null
	 */

	public Job getLastScheduledJob() {
		if (scheduledJobs.size()>0)
			return scheduledJobs.get(scheduledJobs.size()-1);
		else 
			return null;
	}
	
	
	
	
	
	
	
	@Override
	public String toString() {

		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (id: ");
		result.append(id);
		result.append(", currentType: ");
		result.append(currentType);
		result.append(", device: ");
		result.append(device);
		result.append(", instance: ");
		result.append(instance);
		result.append(", machineDown: ");
		for(Interval j: machineDownIntervals)
			result.append(j+", ");
		result.append(')');
		return result.toString();
	}

	

	
	/**
	 * <!-- begin-user-doc -->
	 * This nested class provide an integer comparator to sort the availability list
	 * <!-- end-user-doc -->
	 */
	private static class Interval {
		
		private int from;
		private int to;
		
		public Interval(int value1){
			this.from=value1;
			this.to=value1;
		}
		
		/**
		 * <!-- begin-user-doc -->
		 * This constructor will throw a runtime exception if the lower bound is bigger than the upper bound
		 * <!-- end-user-doc -->
		 */
		public Interval(int value1, int value2){
			if (value1>value2)
				throw new RuntimeException("Cannot instantiate Interval, lower bound "+value1+"must be less or eq than the upperbound "+value2);
			this.from=value1;
			this.to=value2;
		}
		
		
		public int getFrom(){
			return from;
		}
		
		public int getTo(){
			return to;
		}
		
		private boolean sameBoundAs(Interval a){
			return  (to==a.getFrom() || from==a.getTo() || to==a.getTo() || from==a.getFrom() );
		}
		
		private boolean oneBoundInside(Interval a){
			return  (a.to>=from && a.to<=to) || ( a.from>=from && a.from<=to) ||
					(to>=a.from && to<=a.to) || ( from>=a.from && from<=a.to);
		}
		
		public boolean mergableWith(Interval a){
			
			return sameBoundAs(a) || oneBoundInside(a) || a.oneBoundInside(this);
		
		}
		
		/**
		 * <!-- begin-user-doc -->
		 * This method will throw runtime exception if the two intervals are not mergable
		 * <!-- end-user-doc -->
		 */
		public static Interval merge(Interval a, Interval b){
			if (!a.mergableWith(b))
				throw new RuntimeException("Intervals "+a.toString()+" and "+b.toString()+"are not mergable");
			else
				return new Interval(Math.min(a.from, b.from), Math.max(a.to, b.to));
		}
			
			
		@Override
			public boolean equals(Object that){
				  if(this == that) return true;

				  if(!(that instanceof Interval)) 
					  return false;

				  Interval thatA = (Interval)that; 

				  return this.from==thatA.from && this.to == thatA.to;
		   }
	}
			
	
	
	
	

	/**
	 * <!-- begin-user-doc -->
	 * This nested class provide an integer comparator to sort the availability list
	 * <!-- end-user-doc -->
	 */
	private static class IntervalComparator implements Comparator<Interval> {
	    @Override
	    public int compare(Interval a, Interval b) {
	    	if (a.equals(b))
	    		return 0;
	    	if (a.getFrom()==b.getFrom())
	    		return a.getTo() < b.getTo() ? -1 : 1;
	    	
	    	return a.getFrom() < b.getFrom() ? -1 : 1;
	    
	    }
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * This nested class provide a job comparator to sort the scheduledJobs list
	 * <!-- end-user-doc -->
	 */
	private static class JobComparator implements Comparator<Job> {
	    @Override
	    public int compare(Job a, Job b) {
	        return a.getStartsAt() < b.getStartsAt() ? -1 : a.getStartsAt() == b.getStartsAt() ? 0 : 1;
	    }
	}

	/**
	 * <!-- begin-user-doc -->
	 * This nested class provide a jobtype comparator to sort the scheduledJobs list
	 * <!-- end-user-doc -->
	 */
	private static class JobTypeComparator implements Comparator<JobType> {
	    @Override
	    public int compare(JobType a, JobType b) {
	        return a.compareTo(b);
	    }
	}


} //MachineImpl
