package scheduler;

/**
 */


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.LineIterator;

import coders.Constants;
import coders.Encoder;
import coders.Encoder_IncrementalScheduling;
import ascass.solving.CaspSolver;
import scheduler.Changeover;
import checkers.Checker;
import scheduler.Group;
import scheduler.Job;
import scheduler.JobType;
import scheduler.Machine;
import scheduler.Scheduler;
import solvers.Solver;
import splitters.Splitter;
/**
 * <!-- begin-user-doc -->
 * The Scheduler class handles all the methods from the parsing from the edb to the production of the solution. 
 * <!-- end-user-doc -->
 */
public class Scheduler {
	

	protected Splitter splitter;
	
	protected Solver solver;

	protected Checker checker;
	
	
	

	/**
	 *Hashmap where the key is a string with the job id and the value is one instance of the corresponded Job class.
	 */
	protected HashMap<String,Job> jobs;
	
	/**
	 *Hashmap where the key is an integer with the machine id and the value is one instance of the corresponded Machine class.
	 */
	protected LinkedHashMap<Integer,Machine> machines;
	
	/**
	 *Hashmap where the key is an integer and the value is one instance of the Group class.
	 */
	protected HashMap<Integer,Group> groups;
	
	/**
	 *Hashmap where the key is a string that represent the concatenation of two JobTypes and the value is one instance of the corresponded Changeover class
	 *This map is used just in the Infineon case, because in the incremental scheduling problem the changeover is not present.
	 */
	protected HashMap<String, Changeover> changeover;

	protected int currentTime = 0;
	
	protected int maxTime;

	protected int maxPenalty;
	
	
	protected String inputPath;


	
	/**
	 *This constructor parses the input based on the parser choice
	 */
	
	public Scheduler(String path) {
		super();
		this.inputPath=path;
		this.jobs= new HashMap<String,Job>();
		this.machines=new LinkedHashMap<Integer,Machine>();
		this.groups=new HashMap<Integer,Group>();
		this.changeover= new HashMap<String,Changeover>();
				
	}
	
	
	public void parseInput(){
		System.out.println("Enconding Incremental Scheduling input...");
		Encoder e= new Encoder_IncrementalScheduling(this);
		e.encode();
		System.out.println("Enconding Complete");
	}
	
	


	public Splitter getSplitter() {
		if (splitter == null) 
			return null;
		
		return splitter;
	}



	
	public void setSplitter(Splitter newSplitter) {
		splitter = newSplitter;
	}
	
	
	public Solver getSolver() {
		
		return solver;
	}

	
	public void setSolver(Solver value) {
		this.solver=value;
	}
	
	
	
	
	public void setChecker(Checker newChecker) {
		checker = newChecker;
	}
	
	
	
	public Checker getChecker() {
		if (checker == null) 
			return null;
		
		return checker;
	}


	
	public int getMaxTime() {
		return this.maxTime;
		
	}
	

	public void setMaxTime(int parseInt) {
		this.maxTime=parseInt;
		
	}
	
	

	
	
	public int getMaxPenalty() {
		return this.maxPenalty;
		
	}

	public void setMaxPenalty(int parseInt) {
		this.maxPenalty=parseInt;
		
	}

	public String getInputPath() {
		return inputPath;
	}

	
	


	public HashMap<String,Job> getJobs() {
		if (jobs == null) 
			return null;
		
		return jobs;
	}


	public LinkedHashMap<Integer,Machine> getMachines() {
		if (machines == null) 
			return null;
		
		return machines;
	}


	/**
	 * gets a Machine using device + instance information instead of the machineID
	 * Useful in case of incremental scheduling encoding
	 * @param device Integer that identifies the type of machine
	 * @param instance Integer that identifies the instance number of a certain device
	 * @return The Machine correspondent to the device, instance information
	 */
	public Machine getMachineFromDeviceInstance(String device, int instance) {
		List <Machine> machineOfThatDevice= new ArrayList<Machine>();
		for (Machine m: machines.values())
			if (m.getCurrentType().equals(new JobType(device)))
				machineOfThatDevice.add(m);	
		return machineOfThatDevice.get(instance-1);
		
	}
	
	public HashMap<Integer,Group> getGroups() {
		if (groups == null) 
			groups = null;
		
		return groups;
	}

	
	
	
	public HashMap<String, Changeover> getChangeover() {
		if (changeover == null) 
			return null;
	
		return changeover;
	}

	
	


	


	public int getCurrentTime() {
		return currentTime;
	}


	public void setCurrentTime(int newCurrentTime) {
		currentTime = newCurrentTime;
	}


	
	public int getLotsCount(){
		Set<Integer> alreadyCounted= new HashSet<Integer>(); 
		int count=0;
		for (Job j: getJobs().values())
			if (!alreadyCounted.contains(j.getLot())){
				alreadyCounted.add(j.getLot());
				count++;
			}
		return count;
	}
	
	

	
		
	
	

	public String toString() {


		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (currentTime: ");
		result.append(currentTime);
		result.append(')');
		return result.toString();
	}

	
	
	
	
	
	public void addGroup(Group g) {
		if (groups==null)
			groups=new HashMap<Integer,Group>();
		groups.put(g.getId(), g);
		
	}
	
	
	
	public void PrintGroups(){
		for (Group g : getGroups().values())
			System.out.println(g.toString());
	}
	

	public void updateChangeoverInGroup(Group g) {
		List<JobType> typesInGroup= g.getTypesInGroup();
		for (Machine m : g.getMachinesInGroup())
			if (!typesInGroup.contains(m.getCurrentType()))
				for (Job j: g.getJobsInGroup()){
					Changeover c=getChangeover().get(Changeover.generateKey(j.getType(), m.getCurrentType()));
					if (c!=null)
						g.getChangeover().add(c);
					//	System.out.println("changeover added group "+getId()+"= "+Constants.CHANGEOVER+"("+c.machine+","+c.type1+","+c.type2+","+c.length+").");
					}
	}
		
	
	
	
	


	public int getLatestFinishTime() {
		// TODO Auto-generated method stub
		int max=currentTime;
		for (Machine m: getMachines().values())
			if(m.getLastScheduledJob()!=null && m.getLastScheduledJob().getFinishesAt() > max)
				max=m.getLastScheduledJob().getFinishesAt();
		
		return max;
				
	
	}

	
	
	

	public int getInitialTardiness() {
		int tardiness = 0;
		for (Job j : jobs.values()) {
			// if the currentTime is bigger than the deadline, it means that we
			// are late
			if (!j.isAlreadyFinished()  && j.hasDeadline())
				tardiness += Math.max(currentTime - j.getDeadline(),
						0);
		}
		return tardiness;
	}
	
	
	public int getTardiness() {
		int tardiness = 0;
		for (Job j : jobs.values()) {
			// if the currentTime is bigger than the deadline, it means that we
			// are late
			if (!j.isAlreadyFinished()&& j.hasDeadline())
				tardiness += Math.max(j.getFinishesAt()
						- j.getDeadline(), 0);
		}
		return tardiness;
	}
	
	
	public int getMakeSpan(){
		return getLatestFinishTime()-currentTime;
	}

	
	

	
	public int calculateIdleTime(){
		int finalPoint=0;
		int sumJobsLength=0;
		int sumIdle=0;
		for (Machine m : machines.values()) {
			// if the currentTime is bigger than the deadline, it means that we
			// are late
			for (Job j : m.getScheduledJobs()){
				
				if(j.getFinishesAt() > finalPoint)
					finalPoint= j.getFinishesAt();
				//System.out.println("final: machine "+m.getId()+" job "+j.getId()+"="+finalPoint);
				sumJobsLength+=j.getJobLength().get(m);//gets the length of the job in the machine in which it is assigned
				//System.out.println("sumJobsLength: machine "+m.getId()+" job "+j.getId()+"="+sumJobsLength);
			}
			if (finalPoint!=0)//means that there are jobs in that machine
				sumIdle+=finalPoint-sumJobsLength-currentTime;
			//re-initialize for the next machine
			finalPoint=0;
			sumJobsLength=0;
		}
		return sumIdle;
		
	}
	
	
	public int calculateChangeover(){
		int sum=0;
		for (Machine m : machines.values()) 
			for (Job j : m.getScheduledJobs())
				//finish and start comprehend the changeover time 
				if(!j.isCurrentJob() && !j.isAlreadyFinished() )
					sum+=j.getFinishesAt()-j.getStartsAt()-j.getJobLength().get(m);
				
		return sum;
		
	}
	
	
	
	/**
	 * calculates the sum of all the job length in the machine they are assigned to
	 * @generated
	 */
	public double calculateLengthSum(){
		double sumJobsLength=0;
			for (Job j : jobs.values())
				if (!j.isAlreadyFinished())
					sumJobsLength+=j.getJobLength().get(j.getAssignedToMachine());
			return sumJobsLength;
		
	}
	
	/**
	 * calculates the sum of all the job duration in the machine they are assigned to
	 * the duration is the length of the job + the changeover (if present)
	 * @generated
	 */
	public double calculateDurationSum(){
		
		double sumJobsLength=0;
			for (Job j : jobs.values())
				if (!j.isAlreadyFinished())
					sumJobsLength+=j.getFinishesAt()-j.getStartsAt();
			return sumJobsLength;
		
	}

	public void generateTheoreticalMaxTime() {
		
		int sum= 0;
		for (Job j : jobs.values())
			sum+= j.getMaxJobLength();
		
		maxTime=sum;
		
	}
	
	
	public void printMachines(){

		for (Machine m : machines.values()) 
			System.out.println(m.toString());
	
	}
	
	public void printAllContent(){
		System.out.println("Currenttime= " +currentTime);
		System.out.println("Maxtime= " +maxTime);
		System.out.println("JOBS");
		System.out.println("_____________________________");
		for (Job j : jobs.values())
			System.out.println(j.toString());
		System.out.println("MACHINES");
		System.out.println("_____________________________");
		for (Machine m : machines.values()) 
			System.out.println(m.toString());
		System.out.println("GROUPS");
		System.out.println("_____________________________");
		for (Group m : groups.values()) 
			System.out.println(m.toString());
		System.out.println("CHANGEOVERS");
		System.out.println("_____________________________");
		for (Changeover m : changeover.values()) 
			System.out.println(m.toString());
	}



	
	public void printSolution(){

		for (Machine m : machines.values()) {
			System.out.println("MACHINE "+m.id+","+m.getDevice()+","+m.getInstance()+"\n");
			for (Job j: m.getScheduledJobs())
				System.out.println("    job("+j.id+","+j.getStartsAt()+","+j.getFinishesAt()+").\n");
		}
	}
	
} 
