package solvers;

import java.io.File;
import java.io.IOException;

import org.antlr.v4.runtime.misc.Utils;
import org.apache.commons.io.FileUtils;

import coders.Constants;
import coders.Decoder;
import coders.Decoder_IncrementalScheduling;
import coders.Encoder;
import coders.Encoder_IncrementalScheduling;
import ascass.solving.CaspSolver;
import scheduler.Group;
import scheduler.Job;
import scheduler.Machine;
import scheduler.Scheduler;

public class Cumulative_IncrementalScheduling extends Solver{
	
	private static final String INTERNALSOLVERPATH= "/ascass/solvers/Cumulative_IncrementalScheduling.ascass";
	private Encoder encoder;
	private Decoder decoder;


	public Cumulative_IncrementalScheduling(Scheduler scheduler) {
		super(scheduler);
		encoder=new Encoder_IncrementalScheduling(scheduler);
		decoder=new Decoder_IncrementalScheduling(scheduler);
	}
	
	public Cumulative_IncrementalScheduling(Scheduler scheduler,String heuristic) {
		super(scheduler,heuristic);
		encoder=new Encoder_IncrementalScheduling(scheduler);
		decoder=new Decoder_IncrementalScheduling(scheduler);
	}
	/**
	 * Specifies the scheduler 
	 * @param scheduler
	 * @param timeout specifies the timeout of the solver in seconds
	 */
	public Cumulative_IncrementalScheduling(Scheduler scheduler,int timeout) {
		super(scheduler,timeout);
		encoder=new Encoder_IncrementalScheduling(scheduler);
		decoder=new Decoder_IncrementalScheduling(scheduler);
	}

	
	
	
	@Override
	public int solve() {
		System.out.println("solver start");
		String input,program = null;
		String currentFolder = "";
		
		File ascassSolverFile = null;
		File heuristicFile;
		try {
			currentFolder = new File(".").getCanonicalPath();
			ascassSolverFile= new File(currentFolder+INTERNALSOLVERPATH);
			program = FileUtils.readFileToString(ascassSolverFile);
			if (getHeuristic()!=null){
				heuristicFile= new File(currentFolder+Constants.HEURISTICSFOLDER+getHeuristic());
				program += FileUtils.readFileToString(heuristicFile);
				}
			
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		String outputString = currentFolder + Constants.OUTPUTFOLDER;
		int i=0;
		
		StringBuilder decodedInput=new StringBuilder();
		int solution;
		long start=System.currentTimeMillis();
		long current;
		for (Group g : scheduler.getGroups().values()) {
			i++;
	
			input = program;
			decodedInput.append(decoder.decode(g));
			input += decodedInput.toString();
			if(timeout!=-1)
				input+= "csptimeout("+timeout+").\n";
			input+= decoder.decodeSolution();
			if (printInfo)
				System.out.println(input);
			System.out.println("Solving group"+i+"...");
			caspSolver = new CaspSolver(input);

			solution = 0;
			try {
				solution = caspSolver.calcSolution();

			} catch (Exception e) {
				e.printStackTrace();
			}

			// if (solver.getStore() != null)
			switch (solution) {
			case -2:
				System.out.println("no solution found:" + solution);
				return -2;
			case -1:
				System.out.println("no solution found:" + solution);
				return -1;
			case 0:
				System.out.println("Unsatisfiable");
				return 0;
			case 1:
				System.out.println("Satisfiable");
				encoder.encodeSolution(caspSolver.getStore().vars);
				
			}
		}
		current=System.currentTimeMillis();
		if (timeout!=-1 && ((current-start)> timeout*1000))
			return -2;
		else
			return 1;
	}


}
