/**
 */
package solvers;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.LineIterator;

import coders.Constants;
import ascass.solving.CaspSolver;
import scheduler.Group;
import scheduler.Job;
import scheduler.Machine;
import scheduler.Scheduler;

// TODO: Auto-generated Javadoc
/**
 * The abstract Class Solver. 
 * This 
 */
public abstract class Solver{
	

	/** The scheduler to which the solver is attached. A solver can be attached to one Scheduler*/
	protected Scheduler scheduler;

	private String heuristic = null; // if no specified, there is no heuristic
											

	/** The Casp solver used in the ASCASS implementation of the jacob code. */
	protected CaspSolver caspSolver;
	
	/** The timeout for the computation of the solver. The default value is -1*/
	protected int timeout=-1;

	protected boolean printInfo;



	public Solver(Scheduler scheduler){
		
		this.scheduler=scheduler;
		
	}


	public Solver(Scheduler scheduler, String heuristic){
		
		this.scheduler=scheduler;
		try {
			setHeuristic(heuristic);
		} catch (NullPointerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	
	public Solver(Scheduler scheduler, int timeout){
		
		this.scheduler=scheduler;
		this.timeout=timeout;
		
	}
	
	/**
	 * Gets the scheduler binded with this Solver
	 *
	 * @return the scheduler
	 */
	public Scheduler getScheduler() {
		return scheduler;
	}





	/**
	 * This method must be implemented for every class that extends the {@link Solver Solver class}
	 *
	 * @return the int
	 */
	public abstract int solve();


	
	









	/**
	 * Gets the filename of the file that specifies the heuristic used by the solver. 
	 * The filename comprehends the file extension
	 * 
	 *
	 * @return the filename of the heuristic. If there is no heuristic, returns null
	 */
	public String getHeuristic() {
		return heuristic;
	}

	/**
	 * Sets the heuristic used by the solver.
	 * The file must be placed in the folder "heuristics" in the program directory
	 * @param heuristic must be a string with the name of the file that contains the heuristic, with extension
	 * @throws NullPointerException if the given parameter is null
	 * @throws IOException if the given parameter is not in the "heuristics" folder
	 */
	public void setHeuristic(String heuristic) throws IOException,NullPointerException {

		if (heuristic==null)
			throw new NullPointerException();
		
		File f = new File((new File(".").getCanonicalPath())
				+ Constants.HEURISTICSFOLDER);
		
		boolean heuristicInFolder=false;
		for (File j : f.listFiles())
			if (j.isFile() && j.getName().equals(heuristic)) 
				heuristicInFolder=true;
		
		if (heuristicInFolder)
			this.heuristic = heuristic;
		else 
			throw new IOException("Heuristic \""+heuristic+"\" is not present in folder:"+f.getAbsolutePath());
	}


	/**
	 * Gets the casp solver used by the ASCASS implementation. 
	 *
	 * @return the casp solver
	 */
	public CaspSolver getCaspSolver() {
		return caspSolver;
	}
	
	

	public void setPrintInfo(boolean printInfo) {
		this.printInfo = printInfo;
	}

} // SolverImpl
