package splitters;

import java.io.File;
import java.io.IOException;

import org.antlr.v4.runtime.misc.Utils;
import org.apache.commons.io.FileUtils;

import coders.Constants;
import coders.Decoder;
import coders.Decoder_IncrementalScheduling;
import coders.Encoder;
import coders.Encoder_IncrementalScheduling;
import ascass.solving.CaspSolver;
import scheduler.Scheduler;

public class Machinewise_IncrementalScheduling extends Splitter{
	
	
	private static final String INTERNALSPLITTERPATH= "/ascass/splitters/Machine_IncrementalSchedulingPerJob.ascass";
	
	private Encoder encoder;
	private Decoder decoder;


	

	public Machinewise_IncrementalScheduling(Scheduler scheduler) {
		super(scheduler);
		encoder=new Encoder_IncrementalScheduling(scheduler);
		decoder=new Decoder_IncrementalScheduling(scheduler);
	}

	@Override
	public int split() {
		System.out.println("splitter start");
		String input,program = null;
		String currentFolder = "";
		
		File ascassSolverFile = null;
		try {
			currentFolder = new File(".").getCanonicalPath();
			ascassSolverFile= new File(currentFolder+INTERNALSPLITTERPATH);
			program = FileUtils.readFileToString(ascassSolverFile);
			
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		
	
		int solution;
		
		input = program;
		input +=splittingFactorToString();
		input += decoder.decode();
			

		caspSolver = new CaspSolver(input);

		solution = 0;
		try {
			solution = caspSolver.calcSolution();

			} catch (Exception e) {
				e.printStackTrace();
			}

			// if (solver.getStore() != null)
			switch (solution) {
			case -2:
				System.out.println("no solution found:" + solution);
				return -2;
			case -1:
				System.out.println("no solution found:" + solution);
				return -1;
			case 0:
				System.out.println("Unsatisfiable");
				return 0;
			case 1:
				System.out.println("Satisfiable");
				encoder.encodeGroups(caspSolver.getStore().vars);;
				
			}
		
		return 1;
	}

	
	
	private String splittingFactorToString(){
		if (splittingFactor!=0)
			return "\nsplittingFactor("+splittingFactor+").\n";
		else
			return "";
	}
	

}
