package splitters;


import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.LineIterator;

import coders.Constants;
import ascass.solving.CaspSolver;
import scheduler.Group;
import scheduler.Job;
import scheduler.Machine;
import scheduler.Scheduler;

// TODO: Auto-generated Javadoc
/**
 * The abstract Class Solver. 
 * This 
 */
public abstract class Splitter{
	

	/** The scheduler to which the solver is attached. A solver can be attached to one Scheduler*/
	protected Scheduler scheduler;

	protected int splittingFactor=0;

	/** The Casp solver used in the ASCASS implementation of the jacob code. */
	protected CaspSolver caspSolver;



	public Splitter(Scheduler scheduler, int splittingFactor ){
		
		this.scheduler=scheduler;
		this.splittingFactor=splittingFactor;
	}



	
	public Splitter(Scheduler scheduler) {
		this.scheduler=scheduler;
	}




	/**
	 * Gets the scheduler binded with this Solver
	 *
	 * @return the scheduler
	 */
	public Scheduler getScheduler() {
		return scheduler;
	}




	public int getSplittingFactor() {
		return splittingFactor;
	}


	public void setSplittingFactor(int splittingFactor) {
		this.splittingFactor=splittingFactor;
	}

	/**
	 * This method must be implemented for every class that extends the {@link Splitter Solver class}
	 *
	 * @return the int
	 */
	public abstract int split();


	
	











	/**
	 * Gets the casp solver used by the ASCASS implementation. 
	 *
	 * @return the casp solver
	 */
	public CaspSolver getCaspSolver() {
		return caspSolver;
	}

} // SolverImpl
